  
C***********************************************************************
C   Portions of Models-3/CMAQ software were developed or based on      *
C   information from various groups: Federal Government employees,     *
C   contractors working on a United States Government contract, and    *
C   non-Federal sources (including research institutions).  These      *
C   research institutions have given the Government permission to      *
C   use, prepare derivative works, and distribute copies of their      *
C   work in Models-3/CMAQ to the public and to permit others to do     *
C   so.  EPA therefore grants similar permissions for use of the       *
C   Models-3/CMAQ software, but users are requested to provide copies  *
C   of derivative works to the Government without restrictions as to   *
C   use by others.  Users are responsible for acquiring their own      *
C   copies of commercial software associated with Models-3/CMAQ and    *
C   for complying with vendor requirements.  Software copyrights by    *
C   the MCNC Environmental Modeling Center are used with their         *
C   permissions subject to the above restrictions.                     *
C***********************************************************************

C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
      PROGRAM  DRIVER

C-----------------------------------------------------------------------
C Function:
C     -> CTM driver for 4D-Var DATA ASSIMILATION
C     -> Uses L-BFGS optimization routine
C     -> Exponential Preconditioning option available with this package. 
C        To invoke it, change EXP_PRECOND to 'TRUE', line = 127

C CAUTION:
C     -> Driver designed to perturb and retrieve ozone only
C     -> To perturb other species change SPC index of CGRID in
C        the Initialization, line = 297 & 302
C     -> To retrieve accordingly, change NSPCOPTSTART & NSPCOPTEND
C        variables, line = 318

C USEFUL FILES:
C     CONC_L3CHK = Reference/Base checkpoint file
C     CONC_L5CHK = Initial Guess checkpoint file
C     CONC_CHK   = Next Best Guess concentration
 
C Subroutines and functions called:
C     INITSCEN, ADVSTEP, M3EXIT, WRITE3
C     subdrivers: 
C     SUBDRIVER_FWD     - Forward Run for Reference checkpointing
C     SUBDRIVER_FWDPERT - Forward Run for Best Guess checkpointing
C     SUBDRIVER_BWD     - Forward and Backward Run to calculate cost function
C                         & its gradient at each optimization iteration

C Revision History:
C     Kumaresh Singh & Sandu,A. Apr 07 - Added to perform Finite-Difference 
C     and Sensitivity test
C-----------------------------------------------------------------------

      USE PCGRID_DEFN           ! inherits HGRD_DEFN and CGRID_SPCS
      USE VGRD_DEFN             ! vertical layer specifications
      USE AVG_CONC              ! integral average CONC

      IMPLICIT NONE

C     Include Files:

!     INCLUDE SUBST_VGRD_ID     ! vertical dimensioning parameters
      INCLUDE SUBST_IOPARMS     ! I/O parameters definitions
      INCLUDE SUBST_IODECL      ! I/O definitions and declarations
      INCLUDE SUBST_FILES_ID    ! I/O definitions and declarations
      INCLUDE SUBST_PACTL_ID    ! PA control parameters


C     External Functions (not already declared by IODECL3.EXT):

      LOGICAL, EXTERNAL :: ENVYN ! get environment variable as boolean
      LOGICAL, SAVE     :: FIRSTIME = .TRUE.

C     Load the mechanism COMMON ... (next two INCLUDE files)

      INCLUDE SUBST_RXCMMN      ! chemical mechamism reactions COMMON
      INCLUDE SUBST_RXDATA      ! chemical mechamism reactions DATA
      INCLUDE SUBST_GC_SPC      ! gas chemistry species table

C Local variables:

      INTEGER      LOGDEV       ! FORTRAN unit number for log file
      INTEGER      STDATE       ! starting date,    format YYYYDDD
      INTEGER      STTIME       ! starting time,    format HHMMSS
      INTEGER      TSTEP( 2 )   ! time step vector (HHMMSS)
                                ! TSTEP(1) = local output step
                                ! TSTEP(2) = sciproc sync. step (chem)
!     INTEGER      ASTEP( NLAYS )
      INTEGER      SDATE, STIME
      INTEGER, ALLOCATABLE :: ASTEP( : )
      INTEGER      NSTEPS       ! run duration: number of output time steps
      INTEGER      NREPS        ! number of model time steps per output step
      INTEGER      ISTEP        ! current output time step number
      INTEGER      IREP         ! model step number within this output step
      INTEGER      JDATE        ! current model date, coded YYYYDDD
      INTEGER      JTIME        ! current model time, coded HHMMSS
      INTEGER      I, C, R, K, S, V     ! loop induction variables
      INTEGER      ALLOCSTAT
      INTEGER      SAVETIME,SAVEDATE
      INTEGER      RESDATE, RESTIME

      CHARACTER( 16 ) :: PNAME = '4DVAR_DRIVER'
      CHARACTER( 96 ) :: XMSG = ' '

      REAL, POINTER :: CGRID( :,:,:,: )
      REAL, POINTER :: LGRID( :,:,:,: )
      REAL, POINTER :: CGRIDREF( :,:,:,: )

      DOUBLE PRECISION  f,rms,diff_norm,ref_norm
      DOUBLE PRECISION, ALLOCATABLE :: g(:)
      DOUBLE PRECISION, ALLOCATABLE :: x(:)
      REAL  D,relerr
      REAL, PARAMETER :: eps = 0.001

c-----------------LBFGS Parameters----------------

      INTEGER, PARAMETER :: nmax=321000, mmax=17
      character*60     task,csave
      logical          lsave(4)
      integer          n,m,iprint,nbd(nmax),iwa(3*nmax),isave(44)
      double precision factr,pgtol,l(nmax),u(nmax)
      double precision dsave(29)
      double precision wa(2*mmax*nmax+4*nmax+12*mmax*mmax+12*mmax)
      INTEGER :: N_CMAQ_RUNS = 0
      LOGICAL :: EXP_PRECOND = .FALSE. ! Do exponential preconditioning or not
      INTEGER  NSPCOPT,NSPCOPTSTART,NSPCOPTEND

c------------------------------------------

      INTEGER PAR_ERR     ! Error code from parallel initialization. 
      REAL    CLOCK       ! Wall-clock time (sec) at initialization.

      INTERFACE
         SUBROUTINE INITSCEN 
     &                      ( CGRID, STDATE, STTIME, TSTEP, NSTEPS )
            IMPLICIT NONE
            REAL, POINTER             :: CGRID( :,:,:,: )
            INTEGER, INTENT( OUT )    :: STDATE, STTIME
            INTEGER, INTENT( OUT )    :: TSTEP( 2 )
            INTEGER, INTENT( OUT )    :: NSTEPS
         END SUBROUTINE INITSCEN
         SUBROUTINE PA_INIT ( CGRID, JDATE, JTIME, TSTEP )
            IMPLICIT NONE
            REAL, POINTER             :: CGRID( :,:,:,: )
            INTEGER, INTENT( IN )     :: JDATE, JTIME
            INTEGER, INTENT( IN )     :: TSTEP( 2 )
         END SUBROUTINE PA_INIT
         SUBROUTINE SUBDRIVER_FWD(CGRID, STDATE, STTIME, TSTEP, NSTEPS)
            IMPLICIT NONE
            REAL, POINTER             :: CGRID( :,:,:,: )
            INTEGER                   :: STDATE, STTIME 
            INTEGER                   :: TSTEP( 2 ), NSTEPS            
         END SUBROUTINE SUBDRIVER_FWD
         SUBROUTINE SUBDRIVER_FWDPERT(CGRID, STDATE, STTIME, TSTEP, NSTEPS)
            IMPLICIT NONE
            REAL, POINTER             :: CGRID( :,:,:,: )
            INTEGER                   :: STDATE, STTIME 
            INTEGER                   :: TSTEP( 2 ), NSTEPS            
         END SUBROUTINE SUBDRIVER_FWDPERT
         SUBROUTINE SUBDRIVER_BWD(CGRID, f, LGRID, STDATE, STTIME, TSTEP, NSTEPS)
            IMPLICIT NONE
            REAL, POINTER             :: CGRID( :,:,:,: )
            REAL, POINTER             :: LGRID( :,:,:,: )
            DOUBLE PRECISION          :: f         
            INTEGER                   :: STDATE, STTIME 
            INTEGER                   :: TSTEP( 2 ), NSTEPS            
         END SUBROUTINE SUBDRIVER_BWD
      END INTERFACE

C====================================================================
C     Log Files to save cost function and rms values each model run
C     
C     model_runs.m -> number of model runs performed by L-BFGS routine
C     cost_fun.m   -> cost function values(f) for each (m)
C     rms_value.m  -> rms_i=norm(Ci-C0)/norm(C0), i = 1,2,...,m.
C     
C     ---------Contribution to cost function--------------
C     obs_cost.dat -> due to observation misfit
C     bg_cost.dat  -> due to background
C====================================================================

      OPEN(unit=24,file='cost_fun.m') 
      OPEN(unit=25,file='rms_value.m') 
      OPEN(unit=26,file='model_runs.m') 
      OPEN(unit=41,file='obs_cost.dat')
      OPEN(unit=42,file='bg_cost.dat') 

C Get number of species, and starting indices for CGRID array.

      CALL CGRID_MAP ( NSPCSD, GC_STRT, AE_STRT, NR_STRT, TR_STRT )
      
C Start up processor communication and define horizontal domain decomposition
C and vertical layer structure

      CALL PAR_INIT( PNAME, NSPCSD, CLOCK, PAR_ERR )

      IF ( PAR_ERR .NE. 0 ) THEN
         XMSG = 'Error in PAR_INIT'
         CALL M3EXIT ( PNAME, 0, 0, XMSG, XSTAT2 )
         END IF

      LOGDEV = INIT3 ()

C Initialize PCGRID

      IF ( .NOT. PCGRID_INIT() ) THEN
         XMSG = 'Failure defining horizontal domain'
         CALL M3EXIT ( PNAME, 0, 0, XMSG, XSTAT2  )
         END IF
      
      CGRID => PCGRID(1:MY_NCOLS,1:MY_NROWS,:,:)   ! required for PinG

C Initialize conc field: Copy IC's to CONC file as step 0
C Convention: the input file concentration units are always ppmV.

      CALL INITSCEN ( CGRID, STDATE, STTIME, TSTEP, NSTEPS )

C Verify input file header consistency with COORD.EXT and run duration

      CALL FLCHECK ( STDATE, STTIME, TSTEP( 1 ) )

      JDATE = STDATE
      JTIME = STTIME

      RESDATE = STDATE
      RESTIME = STTIME

      IF ( LIPR .OR. LIRR ) CALL PA_INIT ( CGRID, STDATE, STTIME, TSTEP )


C Allocate memories for dynamic arrays
    
      ALLOCATE ( g( MY_NCOLS*MY_NROWS*NLAYS*NSPCSD ), STAT = ALLOCSTAT )
      IF ( ALLOCSTAT .NE. 0 ) THEN
         XMSG = 'g memory allocation failed'
         CALL M3EXIT ( PNAME, JDATE, JTIME, XMSG, XSTAT1 )
      END IF

      ALLOCATE ( x( MY_NCOLS*MY_NROWS*NLAYS*NSPCSD ), STAT = ALLOCSTAT )
      IF ( ALLOCSTAT .NE. 0 ) THEN
         XMSG = 'x memory allocation failed'
         CALL M3EXIT ( PNAME, JDATE, JTIME, XMSG, XSTAT1 )
      END IF

      ALLOCATE ( CGRIDREF( MY_NCOLS,MY_NROWS,NLAYS,NSPCSD ), STAT = ALLOCSTAT )
      IF ( ALLOCSTAT .NE. 0 ) THEN
         XMSG = 'CGRIDREF memory allocation failed'
         CALL M3EXIT ( PNAME, JDATE, JTIME, XMSG, XSTAT1 )
      END IF

      ALLOCATE ( LGRID( MY_NCOLS,MY_NROWS,NLAYS,NSPCSD ), STAT = ALLOCSTAT )
      IF ( ALLOCSTAT .NE. 0 ) THEN
         XMSG = 'LGRID memory allocation failed'
         CALL M3EXIT ( PNAME, JDATE, JTIME, XMSG, XSTAT1 )
      END IF

C Open Checkpoint Files

      IF ( .NOT. OPEN3( CONC_L2CHK, FSRDWR3, PNAME ) ) THEN
         XMSG = 'Could not open ' // CONC_L2CHK // ' file for update'
      END IF

      IF ( .NOT. OPEN3( CONC_L4CHK, FSRDWR3, PNAME ) ) THEN
         XMSG = 'Could not open ' // CONC_L4CHK // ' file for update'
      END IF

      IF ( .NOT. OPEN3( CONC_L5CHK, FSRDWR3, PNAME ) ) THEN
         XMSG = 'Could not open ' // CONC_L5CHK // ' file for update'
      END IF
       
      IF ( .NOT. OPEN3( CONC_CHK, FSRDWR3, PNAME ) ) THEN
         XMSG = 'Could not open ' // CONC_CHK // ' file for update'
      END IF
      
      IF ( .NOT. OPEN3( CONC_L3CHK, FSRDWR3, PNAME ) ) THEN
         XMSG = 'Could not open ' // CONC_L3CHK // ' file for update'
      END IF

      IF ( .NOT. OPEN3( ADJFAC_CHK, FSRDWR3, PNAME ) ) THEN
         XMSG = 'Could not open ' // CONC_CHK // ' file for update'
      END IF

      IF ( .NOT. OPEN3( CONC_XADVCHK, FSRDWR3, PNAME ) ) THEN
         XMSG = 'Could not open ' // CONC_CHK // ' file for update'
      END IF

      IF ( .NOT. OPEN3( CONC_YADVCHK, FSRDWR3, PNAME ) ) THEN
         XMSG = 'Could not open ' // CONC_CHK // ' file for update'
      END IF



C Forward run to generate non-constant concentration field

      CALL SUBDRIVER_FWD(CGRID, STDATE, STTIME, TSTEP, NSTEPS)

C====================================================================
C     Resulting CGRID from above forward run treated as the initial 
C     condition for the optimization test
C====================================================================

      CGRIDREF = CGRID

C Forward run to lay down observations for all time steps

      CALL SUBDRIVER_FWD(CGRID, STDATE, STTIME, TSTEP, NSTEPS)

C====================================================================
C     -Introducing Perturbations (SPC = 4 for OZONE)
C     -Perturbation amount = 30.09% of the original concentration
C     -To introduce different perturbation, change the formula 
C      accordingly
C====================================================================

      CGRID = CGRIDREF
      CGRID( :,:,:,4) = CGRID( :,:,:,4)*(1.3+9.0*eps) ! perturbed conc.

C Forward run to lay down initial guess for all time steps

      CALL SUBDRIVER_FWDPERT(CGRID, STDATE, STTIME, TSTEP, NSTEPS)

C Restoring CGRID's value to the initial guess

      CGRID = CGRIDREF
      CGRID( :,:,:,4) = CGRID( :,:,:,4)*(1.3+9.0*eps) ! perturbed conc.

      PRINT*,'NEXT START OPTIMIZATION'

c-------------------------------------------------------------------

c     cccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

c     LBFGS Optimization Steps
c     We wish to have output at every iteration.

      iprint = 1

c     Number of species to optimize

      NSPCOPTSTART = 4
      NSPCOPTEND = 4
      NSPCOPT = NSPCOPTEND-NSPCOPTSTART+1

c-----------------reference norm calculations----------------------

      ref_norm = 0.d0
      
      DO V = NSPCOPTSTART, NSPCOPTEND
         DO K = 1, NLAYS
            DO R = 1, MY_NROWS
               DO C = 1, MY_NCOLS
                  ref_norm = ref_norm+CGRIDREF(C,R,K,V)*CGRIDREF(C,R,K,V)
               END DO
            END DO
         END DO
      END DO

      ref_norm = dsqrt(ref_norm)
         
c------------------------------------------------------------------

c     We specify the tolerances in the stopping criteria.

      factr=1.0d+7
      pgtol=1.0d-7

c     We specify the dimension n of the sample problem and the number
c        m of limited memory corrections stored.  (n and m should not
c        exceed the limits nmax and mmax respectively.)
 
      n = MY_NCOLS*MY_NROWS*NLAYS*NSPCOPT

      m = 5
 
c     We now provide nbd which defines the bounds on the variables:
c                    l   specifies the lower bounds,
c                    u   specifies the upper bounds. 
 
c     First set bounds on variables.

      IF (EXP_PRECOND) THEN
        do i=1,n
           nbd(i)=0   
        end do
      ELSE
        do i=1,n
           nbd(i)=1
           l(i)=0.0d0
        end do
      END IF 


c     We now define the starting point.

c     Convert concentrations to log-concentrations
         DO V = NSPCOPTSTART, NSPCOPTEND
            DO K = 1, NLAYS
               DO R = 1, MY_NROWS
                  DO C = 1, MY_NCOLS
                     IF (EXP_PRECOND) THEN
                        x((((C-1)*MY_NROWS+R-1)*NLAYS+K-1)*NSPCOPT+V-NSPCOPTSTART+1)= LOG( MAX( CGRID(C,R,K,V), 1.0d-16 ) )
                     ELSE
                        x((((C-1)*MY_NROWS+R-1)*NLAYS+K-1)*NSPCOPT+V-NSPCOPTSTART+1)= CGRID(C,R,K,V)
                     END IF
                END DO
               END DO
            END DO
         END DO

      write (6,16)
 16   format(/,5x, 'Solving sample problem.',
     +       /,5x, ' (f = 0.0 at the optimal solution.)',/)

c     We start the iteration by initializing task.
c 
      task = 'START'

c        ------- the beginning of the loop ----------
 
 111  continue
      
c     This is the call to the L-BFGS-B code.
      
      call setulb(n,m,x,l,u,nbd,f,g,factr,pgtol,wa,iwa,task,iprint,
     +     csave,lsave,isave,dsave)
      
      if (task(1:2) .eq. 'FG') then

c       Convert log-concentrations to concentrations
         DO V = NSPCOPTSTART, NSPCOPTEND
            DO K = 1, NLAYS
               DO R = 1, MY_NROWS
                  DO C = 1, MY_NCOLS
                    IF (EXP_PRECOND) THEN
                      CGRID(C,R,K,V) = EXP( x((((C-1)*MY_NROWS+R-1)*NLAYS+K-1)*NSPCOPT+V-NSPCOPTSTART+1) )
                    ELSE
                      CGRID(C,R,K,V) =      x((((C-1)*MY_NROWS+R-1)*NLAYS+K-1)*NSPCOPT+V-NSPCOPTSTART+1) 
                    END IF
                  END DO
               END DO
            END DO
         END DO  

C      calculate cost function(f) value and update LGRID
         CALL SUBDRIVER_BWD(CGRID, f, LGRID, STDATE, STTIME, TSTEP, NSTEPS)
         N_CMAQ_RUNS = N_CMAQ_RUNS + 1

c      Convert adjoint to gradient w.r.t. log-concentrations
         DO V = NSPCOPTSTART, NSPCOPTEND
            DO K = 1, NLAYS
               DO R = 1, MY_NROWS
                  DO C = 1, MY_NCOLS
                    IF (EXP_PRECOND) THEN
                      g((((C-1)*MY_NROWS+R-1)*NLAYS+K-1)*NSPCOPT+V-NSPCOPTSTART+1)= LGRID(C,R,K,V)*CGRID(C,R,K,V)
                    ELSE
                      g((((C-1)*MY_NROWS+R-1)*NLAYS+K-1)*NSPCOPT+V-NSPCOPTSTART+1)= LGRID(C,R,K,V)
                   END IF
                END DO
               END DO
            END DO
         END DO 

c--------------------------------------------
c         RMS calculations
c--------------------------------------------

         diff_norm = 0.d0

         DO V = NSPCOPTSTART, NSPCOPTEND
            DO K = 1, NLAYS
               DO R = 1, MY_NROWS
                  DO C = 1, MY_NCOLS
                     diff_norm = diff_norm+(CGRIDREF(C,R,K,V)-CGRID(C,R,K,V))*(CGRIDREF(C,R,K,V)-CGRID(C,R,K,V))
                  END DO
               END DO
            END DO
         END DO         

         diff_norm = dsqrt(diff_norm)

         rms = diff_norm/ref_norm

c--------------------------------------------
c     checkpointing f, rms and model_runs
c--------------------------------------------
         
         write(24,*) f
         write(25,*) rms
         write(26,*) N_CMAQ_RUNS
c--------------------------------------------
 
         IF (N_CMAQ_RUNS > 26) GOTO 222    !set the number of model runs required was 15 before
         
c     go back to the minimization routine.
         goto 111
      
      elseif (task(1:5) .eq. 'NEW_X')  then
c     the minimization routine has returned with a new iterate,
c     and we have opted to continue the iteration.
         DO V = NSPCOPTSTART, NSPCOPTEND
            DO K = 1, NLAYS
               DO R = 1, MY_NROWS
                  DO C = 1, MY_NCOLS
                    IF (EXP_PRECOND) THEN
                      CGRID(C,R,K,V) = EXP( x((((C-1)*MY_NROWS+R-1)*NLAYS+K-1)*NSPCOPT+V-NSPCOPTSTART+1) )
                    ELSE
                      CGRID(C,R,K,V) =      x((((C-1)*MY_NROWS+R-1)*NLAYS+K-1)*NSPCOPT+V-NSPCOPTSTART+1) 
                    END IF
                  END DO
               END DO
            END DO
         END DO

c     go back to the minimization routine.
         goto 111

      end if
c     ---------- the end of the loop -------------
      
c     If task is neither FG nor NEW_X we terminate execution.
      
c     generating output files------------x----------------

c     ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

 222  CONTINUE

c--------closing checkpointing files-----------------------------


      IF ( .NOT. CLOSE3( CONC_L2CHK ) ) THEN
         XMSG = 'Could not close ' // CONC_L2CHK
         CALL M3WARN( PNAME, STDATE, STTIME, XMSG )
      END IF

      IF ( .NOT. CLOSE3( CONC_L4CHK ) ) THEN
         XMSG = 'Could not close ' // CONC_L4CHK
         CALL M3WARN( PNAME, STDATE, STTIME, XMSG )
      END IF

      IF ( .NOT. CLOSE3( CONC_L5CHK ) ) THEN
         XMSG = 'Could not close ' // CONC_L4CHK
         CALL M3WARN( PNAME, STDATE, STTIME, XMSG )
      END IF

      IF ( .NOT. CLOSE3( CONC_CHK ) ) THEN
         XMSG = 'Could not close ' // CONC_CHK
         CALL M3WARN( PNAME, STDATE, STTIME, XMSG )
      END IF

      IF ( .NOT. CLOSE3( CONC_L3CHK ) ) THEN
         XMSG = 'Could not close ' // CONC_L3CHK
         CALL M3WARN( PNAME, STDATE, STTIME, XMSG )
      END IF

      IF ( .NOT. CLOSE3( ADJFAC_CHK ) ) THEN
         XMSG = 'Could not close ' // ADJFAC_CHK
         CALL M3WARN( PNAME, STDATE, STTIME, XMSG )
      END IF

      IF ( .NOT. CLOSE3( CONC_XADVCHK ) ) THEN
         XMSG = 'Could not close ' //CONC_XADVCHK
         CALL M3WARN( PNAME, STDATE, STTIME, XMSG )
      END IF

      IF ( .NOT. CLOSE3( CONC_YADVCHK ) ) THEN
         XMSG = 'Could not close ' //CONC_YADVCHK
         CALL M3WARN( PNAME, STDATE, STTIME, XMSG )
      END IF


      
c----------------------------------------------------------------

      DEALLOCATE ( g, STAT = ALLOCSTAT )
      IF ( ALLOCSTAT .NE. 0 ) THEN
         XMSG = 'g memory deallocation failed'
         CALL M3EXIT ( PNAME, JDATE, JTIME, XMSG, XSTAT1 )
      END IF

      DEALLOCATE ( x, STAT = ALLOCSTAT )
      IF ( ALLOCSTAT .NE. 0 ) THEN
         XMSG = 'x memory deallocation failed'
         CALL M3EXIT ( PNAME, JDATE, JTIME, XMSG, XSTAT1 )
      END IF




C     Clean up communications.

      CALL PAR_TERM ( PNAME, JDATE, JTIME, CLOCK )

C     Close log files
      CLOSE(24)
      CLOSE(25)
      CLOSE(26)
      CLOSE(41)
      CLOSE(42)
      
      END
