   
C***********************************************************************
C   Portions of Models-3/CMAQ software were developed or based on      *
C   information from various groups: Federal Government employees,     *
C   contractors working on a United States Government contract, and    *
C   non-Federal sources (including research institutions).  These      *
C   research institutions have given the Government permission to      *
C   use, prepare derivative works, and distribute copies of their      *
C   work in Models-3/CMAQ to the public and to permit others to do     *
C   so.  EPA therefore grants similar permissions for use of the       *
C   Models-3/CMAQ software, but users are requested to provide copies  *
C   of derivative works to the Government without restrictions as to   *
C   use by others.  Users are responsible for acquiring their own      *
C   copies of commercial software associated with Models-3/CMAQ and    *
C   for complying with vendor requirements.  Software copyrights by    *
C   the MCNC Environmental Modeling Center are used with their         *
C   permissions subject to the above restrictions.                     *
C***********************************************************************

C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
      SUBROUTINE BG_GRAD_UPDATE ( CGRID, LGRID, CF, JDATE, JTIME )

C-----------------------------------------------------------------------
C Function:
C     Calculates the background cost-function value and updates
C     adjoint variable for initial time 

C CAUTION: 
C     ->This subroutine is being constructed for current test problem. 
C     ->One needs to modify the formulas according to his/her needs.
 
C INPUT:
C     CGRID - Current concentration field (read from CONC_CHK)
C     LGRID - Adjoint variable to be updated
C     CF    - Cost-Function update variable
C     JDATE,JTIME - Current date and time step values

C DATA READ FROM FILES:
C     CBGRID - background(perturbed) conc from file: CONC_L5CHK

C OUTPUT:
C     LGRID,CF
 
C Revision History:
C     Kumaresh Singh & Sandu,A. Apr 07 - Added to calculate background 
C     cost-function and update adjoint variable for model-observations
C-----------------------------------------------------------------------

      USE GRID_CONF             ! horizontal & vertical domain specifications
      USE CGRID_SPCS            ! CGRID species number and offsets

      IMPLICIT NONE   

      INCLUDE SUBST_IOPARMS     ! I/O parameters definitions
      INCLUDE SUBST_IODECL      ! I/O definitions and declarations
      INCLUDE SUBST_FILES_ID    ! I/O definitions and declarations
      INCLUDE SUBST_PACTL_ID    ! PA control parameters

C Arguments:

      REAL, POINTER :: CGRID( :,:,:,: ) ! concentrations
      REAL, POINTER :: LGRID( :,:,:,: )
      REAL             CF

C Variables

      REAL, POINTER :: CBGRID( :,:,:,: )
      REAL, ALLOCATABLE :: SIGMA( :,: )
      REAL        D
      INTEGER     I,J,K,L,istatus    ! loop counters

      CHARACTER( 16 ) :: PNAME = 'BG_GRAD_UPDATE'
      CHARACTER( 96 ) :: XMSG = ' '

      INTEGER      JDATE        ! current model date, coded YYYYDDD
      INTEGER      JTIME        ! current model time, coded HHMMSS
      INTEGER      ALLOCSTAT

      INTERFACE
         SUBROUTINE RD_L5CHK ( CBGRID, JDATE, JTIME )
         IMPLICIT NONE
         REAL, POINTER             :: CBGRID( :,:,:,: )
         INTEGER, INTENT( IN )     :: JDATE, JTIME
         END SUBROUTINE RD_L5CHK   
      END INTERFACE

C-----------------------------------------------------------------------

C     Initialization

      CF   = 0d0

      ALLOCATE ( CBGRID( MY_NCOLS,MY_NROWS,NLAYS,NSPCSD ), STAT = ALLOCSTAT )
      IF ( ALLOCSTAT .NE. 0 ) THEN
         XMSG = 'CBGRID memory allocation failed'
         CALL M3EXIT ( PNAME, JDATE, JTIME, XMSG, XSTAT1 )
      END IF  
      
      ALLOCATE ( SIGMA( NLAYS,NSPCSD ), STAT = ALLOCSTAT )
      IF ( ALLOCSTAT .NE. 0 ) THEN
         XMSG = 'SIGMA memory allocation failed'
         CALL M3EXIT ( PNAME, JDATE, JTIME, XMSG, XSTAT1 )
      END IF  
      
C     ============================================
C     Reading background concentration field
C     ============================================
      CALL RD_L5CHK ( CBGRID, JDATE, JTIME )  
    
C     =========================================================================
C     Compute the background std of each species K at each level L as the 
C     mean background concentration in the entire horizontal layer
C                     std = weight*ave_{ncols,nrows}(CGRID())
C     =========================================================================
      DO K=4,4
         DO L=1,NLAYS
            SIGMA(L,K) = 0d0
            DO J=1,MY_NROWS
               DO I=1,MY_NCOLS
                  SIGMA(L,K) = SIGMA(L,K) + CBGRID(I,J,L,K)
               END DO
            END DO
            SIGMA(L,K) = 12*SIGMA(L,K)/(MY_NROWS*MY_NCOLS)  ! 12 was replaced by 1.2
         END DO
      END DO

C     =========================================================================
C     Calculate the cost function and update adjoint variable as:
C     -> Variance = std**2 
C     -> costfunc = sum_{ncols,nrows,nlays,nspec}(C-Cb)**2/(2*Variance)
C     -> lambda(i,j,l,k) = lambda(i,j,l,k) + (C-Cb)/Variance
C     =========================================================================
      DO K=4,4
         DO L=1,NLAYS
            DO J=1,MY_NROWS
               DO I=1,MY_NCOLS
                  D = (CGRID(I,J,L,K)-CBGRID(I,J,L,K))/SIGMA(L,K)
                  CF = CF + D*D/2
                  LGRID(I,J,L,K) = LGRID(I,J,L,K) + D/SIGMA(L,K)
               END DO
            END DO
         END DO
      END DO
      
      
      DEALLOCATE ( CBGRID, STAT = ALLOCSTAT )
      IF ( ALLOCSTAT .NE. 0 ) THEN
         XMSG = 'CBGRID memory deallocation failed'
         CALL M3EXIT ( PNAME, JDATE, JTIME, XMSG, XSTAT1 )
      END IF  
      
      DEALLOCATE ( SIGMA, STAT = ALLOCSTAT )
      IF ( ALLOCSTAT .NE. 0 ) THEN
         XMSG = 'SIGMA memory deallocation failed'
         CALL M3EXIT ( PNAME, JDATE, JTIME, XMSG, XSTAT1 )
      END IF

      

      END



