C***********************************************************************
C   Portions of Models-3/CMAQ software were developed or based on      *
C   information from various groups: Federal Government employees,     *
C   contractors working on a United States Government contract, and    *
C   non-Federal sources (including research institutions).  These      *
C   research institutions have given the Government permission to      *
C   use, prepare derivative works, and distribute copies of their      *
C   work in Models-3/CMAQ to the public and to permit others to do     *
C   so.  EPA therefore grants similar permissions for use of the       *
C   Models-3/CMAQ software, but users are requested to provide copies  *
C   of derivative works to the Government without restrictions as to   *
C   use by others.  Users are responsible for acquiring their own      *
C   copies of commercial software associated with Models-3/CMAQ and    *
C   for complying with vendor requirements.  Software copyrights by    *
C   the MCNC Environmental Modeling Center are used with their         *
C   permissions subject to the above restrictions.                     *
C***********************************************************************

C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
      SUBROUTINE OBS_GRAD_UPDATE ( CGRID, LGRID, CF, JDATE, JTIME )

C-----------------------------------------------------------------------
C Function:
C     Calculates the observation cost-function update value and updates
C     adjoint variable for the current dynamic time-step

C CAUTION: 
C     This subroutine uses model-observations to calculate cost function
C     and its gradient. One needs to modify the formulaes and I/O procedures
C     to incorporate real-observations.
 
C INPUT:
C     CGRID - Current concentration field
C     LGRID - Adjoint variable to be updated
C     CF    - Cost-Function update variable
C     JDATE,JTIME - Current date and time step values

C DATA READ FROM FILES:
C     CHKGRID - observed(reference) conc from file: CONC_L3CHK

C OUTPUT:
C     LGRID,CF
 
C Revision History:
C     Kumaresh Singh & Sandu,A., Apr 07 - Added to calculate observation 
C     cost-function and update adjoint variable for model-observations
C-----------------------------------------------------------------------

      USE GRID_CONF             ! horizontal & vertical domain specifications
      USE CGRID_SPCS            ! CGRID species number and offsets

      IMPLICIT NONE   

      INCLUDE SUBST_IOPARMS     ! I/O parameters definitions
      INCLUDE SUBST_IODECL      ! I/O definitions and declarations
      INCLUDE SUBST_FILES_ID    ! I/O definitions and declarations
      INCLUDE SUBST_PACTL_ID    ! PA control parameters

C Arguments:

      REAL, POINTER :: CGRID( :,:,:,: ) ! concentrations
      REAL, POINTER :: LGRID( :,:,:,: )
      REAL             CF

C Variables

      REAL, POINTER :: CHKGRID( :,:,:,: )
      REAL, POINTER :: AVE( :,: )
      REAL, POINTER :: NOBS( :,: )
      REAL        D
      INTEGER     m,n,o,p,oprev,istatus     ! loop counters
     
      REAL, PARAMETER :: UNC_OBS = 0.08   !2.5 ! orginal value was 0.08
      
      CHARACTER( 16 ) :: PNAME = 'OBS_GRAD_UPDATE'
      CHARACTER( 96 ) :: XMSG = ' '

      INTEGER      JDATE        ! current model date, coded YYYYDDD
      INTEGER      JTIME        ! current model time, coded HHMMSS
      INTEGER      ALLOCSTAT

      INTERFACE
         SUBROUTINE RD_L3CHK ( CHKGRID, JDATE, JTIME )
         IMPLICIT NONE
         REAL, POINTER             :: CHKGRID( :,:,:,: )
         INTEGER, INTENT( IN )     :: JDATE, JTIME
         END SUBROUTINE RD_L3CHK   
      END INTERFACE

C-----------------------------------------------------------------------

C     Initializations

      CF   = 0d0

      ALLOCATE ( CHKGRID( MY_NCOLS,MY_NROWS,NLAYS,NSPCSD ), STAT = ALLOCSTAT )
      IF ( ALLOCSTAT .NE. 0 ) THEN
         XMSG = 'CHKGRID memory allocation failed'
         CALL M3EXIT ( PNAME, JDATE, JTIME, XMSG, XSTAT1 )
      END IF  

      ALLOCATE ( AVE( NLAYS,NSPCSD ), STAT = ALLOCSTAT )
      IF ( ALLOCSTAT .NE. 0 ) THEN
         XMSG = 'SIGMA memory allocation failed'
         CALL M3EXIT ( PNAME, JDATE, JTIME, XMSG, XSTAT1 )
      END IF  

      ALLOCATE ( NOBS( NLAYS,NSPCSD ), STAT = ALLOCSTAT )
      IF ( ALLOCSTAT .NE. 0 ) THEN
         XMSG = 'SIGMA memory allocation failed'
         CALL M3EXIT ( PNAME, JDATE, JTIME, XMSG, XSTAT1 )
      END IF       
      
      NOBS = 0d0
      AVE  = 0d0

C     ============================================
C     Read observation cell in CHKGRID
C     ============================================
      
      CALL RD_L3CHK ( CHKGRID, JDATE, JTIME )      

C     =========================================================================
C     Compute the observation std of each species K at each level L as 8% of the 
C     mean observation concentration in the observational horizontal layer
C                     std = 0.08*ave_{obscols,obsrows}
C     =========================================================================

C     ----------------------------
C     observation grid read from
C     file 'mask.dat'
C     ----------------------------

      OPEN(UNIT=20,FILE='mask.dat')
      
C     --------------------------------
C     Calculating 
C     sum_{obscols,obsrows}(CHKGRID())
C     --------------------------------

      DO 		     
         READ(UNIT=20,FMT=*,IOSTAT=istatus)  m,n,o,p
         IF (istatus<0) EXIT    ! end of file
         AVE(o,p)  = AVE(o,p) + CHKGRID(m,n,o,p)
         NOBS(o,p) = NOBS(o,p) + 1
      END DO 
      REWIND(20)

C     -----------------------------------------------------------
C     Calculating 
C     -> ave_{obscols,obsrows} = sum_{obscols,obsrows}(CHKGRID())
C     -> std = 0.08*ave_{obscols,obsrows}
C     -----------------------------------------------------------
      oprev = 0
      DO 		     
         READ(UNIT=20,FMT=*,IOSTAT=istatus)  m,n,o,p
         IF (istatus<0) EXIT    ! end of file      
         IF(oprev.ne.o) THEN    ! check if current layer number is different than prev
            AVE(o,p) = AVE(o,p)*UNC_OBS/NOBS(o,p)
         END IF
         IF(AVE(o,p).eq.0)THEN  ! conc on a certain layer might be zero on the obs grid
            AVE(o,p) = 1d0 
         END IF
         oprev = o
      END DO 
      REWIND(20)

C     =========================================================================
C     Calculate cost function and update the adjoint variable as:
C     -> Variance = std**2 
C     -> costfunc = sum_{obscols,obsrows,obslays,obsspec}(C-Cobs)**2/2*Variance
C     -> lambda(c,r,l,s) = lambda(c,r,l,s) + (C-Cobs)/Variance
C     =========================================================================

      DO 		     
         READ(UNIT=20,FMT=*,IOSTAT=istatus)  m,n,o,p
         IF (istatus<0) EXIT    ! end of file
         D  = (CGRID(m,n,o,p) - CHKGRID(m,n,o,p))/AVE(o,p)
         CF = CF + D*D/2
         LGRID(m,n,o,p) = LGRID(m,n,o,p) + D/AVE(o,p)
      END DO 

      CLOSE(20)
      
      
      
      DEALLOCATE ( CHKGRID, STAT = ALLOCSTAT )
      IF ( ALLOCSTAT .NE. 0 ) THEN
         XMSG = 'CHKGRID memory deallocation failed'
         CALL M3EXIT ( PNAME, JDATE, JTIME, XMSG, XSTAT1 )
      END IF  

      DEALLOCATE ( AVE, STAT = ALLOCSTAT )
      IF ( ALLOCSTAT .NE. 0 ) THEN
         XMSG = 'SIGMA memory deallocation failed'
         CALL M3EXIT ( PNAME, JDATE, JTIME, XMSG, XSTAT1 )
      END IF  

      DEALLOCATE ( NOBS, STAT = ALLOCSTAT )
      IF ( ALLOCSTAT .NE. 0 ) THEN
         XMSG = 'SIGMA memory deallocation failed'
         CALL M3EXIT ( PNAME, JDATE, JTIME, XMSG, XSTAT1 )
      END IF 

      END
