    
C***********************************************************************
C   Portions of Models-3/CMAQ software were developed or based on      *
C   information from various groups: Federal Government employees,     *
C   contractors working on a United States Government contract, and    *
C   non-Federal sources (including research institutions).  These      *
C   research institutions have given the Government permission to      *
C   use, prepare derivative works, and distribute copies of their      *
C   work in Models-3/CMAQ to the public and to permit others to do     *
C   so.  EPA therefore grants similar permissions for use of the       *
C   Models-3/CMAQ software, but users are requested to provide copies  *
C   of derivative works to the Government without restrictions as to   *
C   use by others.  Users are responsible for acquiring their own      *
C   copies of commercial software associated with Models-3/CMAQ and    *
C   for complying with vendor requirements.  Software copyrights by    *
C   the MCNC Environmental Modeling Center are used with their         *
C   permissions subject to the above restrictions.                     *
C***********************************************************************

C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
      PROGRAM  DRIVER

C-----------------------------------------------------------------------
C Function:
C     -> CTM driver for FINITE DIFFERENCE TESTS
C     -> FD TEST: Introduce different perturbations and run the adjoint
C                 model twice to calculate respective cost funtions f1 
C                 and f2 and validate the model with the following eqn: 
C                 "Costfunc2-Costfunc1 = LGRID2'.(CGRID2-CGRID1);"

C CAUTION:
C     -> Driver designed to perform FD test with ozone
C     -> To perturb other species change SPC index of CGRID in
C        Finite-Difference Initialization, line = 327

C CHECKPOINT FILES:
C     ->Relevant output Files
C       CONC_L3CHK = Reference/Base checkpoint file
C       CONC_CHK   = Current CGRID
C
C     ->Irrelevant intermediate checkpoint files
C       CONC_L2CHK = CGRID before starting chemistry
C       CONC_L4CHK = Checkpoint air density in forward science process
 
C Subroutines and functions called:
C     INITSCEN, ADVSTEP, M3EXIT, WRITE3
C     science processes SCIPROC, PA_OUTPUT
C     SUBDRIVER_FWD -> lay down observation grid
C     FDDRIVER_BWD  -> calculate cost function
C     read checkpoint file -> RD_CHK
 
C Revision History:
C     Kumaresh Singh & Sandu,A. Apr 07 - Added to perform Finite-Difference 
C     and Sensitivity test
C     Tianyi Gou & Sandu,A.     Nov 12 - open/close new checkpoints files
C-----------------------------------------------------------------------

      USE PCGRID_DEFN           ! inherits HGRD_DEFN and CGRID_SPCS
      USE VGRD_DEFN             ! vertical layer specifications
      USE AVG_CONC              ! integral average CONC

      IMPLICIT NONE

C Include Files:

!     INCLUDE SUBST_VGRD_ID     ! vertical dimensioning parameters
      INCLUDE SUBST_IOPARMS     ! I/O parameters definitions
      INCLUDE SUBST_IODECL      ! I/O definitions and declarations
      INCLUDE SUBST_FILES_ID    ! I/O definitions and declarations
      INCLUDE SUBST_PACTL_ID    ! PA control parameters

C Load the mechanism COMMON ... (next two INCLUDE files)
      INCLUDE SUBST_RXCMMN      ! chemical mechamism reactions COMMON
      INCLUDE SUBST_RXDATA      ! chemical mechamism reactions DATA
      INCLUDE SUBST_GC_SPC      ! gas chemistry species table

C External Functions (not already declared by IODECL3.EXT):

      LOGICAL, EXTERNAL :: ENVYN ! get environment variable as boolean
      LOGICAL, SAVE     :: FIRSTIME = .TRUE.

C Local variables:

      INTEGER      LOGDEV       ! FORTRAN unit number for log file
      INTEGER      STDATE       ! starting date,    format YYYYDDD
      INTEGER      STTIME       ! starting time,    format HHMMSS
      INTEGER      TSTEP( 2 )   ! time step vector (HHMMSS)
                                ! TSTEP(1) = local output step
                                ! TSTEP(2) = sciproc sync. step (chem)
!     INTEGER      ASTEP( NLAYS )
      INTEGER      SDATE, STIME
      INTEGER, ALLOCATABLE :: ASTEP( : )
      INTEGER      NSTEPS       ! run duration: number of output time steps
      INTEGER      NREPS        ! number of model time steps per output step
      INTEGER      ISTEP        ! current output time step number
      INTEGER      IREP         ! model step number within this output step
      INTEGER      JDATE        ! current model date, coded YYYYDDD
      INTEGER      JTIME        ! current model time, coded HHMMSS
      INTEGER      C, R, L, K, S, V     ! loop induction variables
      INTEGER      ALLOCSTAT
      INTEGER      SAVETIME,SAVEDATE
      INTEGER      RESDATE, RESTIME

      CHARACTER( 16 ) :: PNAME = 'FD_DRIVER'
      CHARACTER( 96 ) :: XMSG = ' '

      REAL, POINTER :: CGRID( :,:,:,: )
c-----------------------------------------
      REAL, POINTER :: CGRIDREF( :,:,:,: )
      REAL, POINTER :: FIRSTPERT( :,:,:,: )
      REAL, POINTER :: LGRID( :,:,:,: )

      DOUBLE PRECISION  f,f1,sum
      REAL  D,relerr,CF
      REAL, PARAMETER :: eps = 0.001
      INTEGER  m,n,o,p,istatus,I
      INTEGER  SPB,SPE
c------------------------------------------

      REAL, ALLOCATABLE :: AGRID( :,:,:,: )
      REAL    DIVFAC      ! trapezoidal average factor
      INTEGER A_NLYS

      INTEGER PAR_ERR     ! Error code from parallel initialization. 
      REAL    CLOCK       ! Wall-clock time (sec) at initialization.

      INTERFACE
         SUBROUTINE INITSCEN 
     &                      ( CGRID, STDATE, STTIME, TSTEP, NSTEPS )
            IMPLICIT NONE
            REAL, POINTER             :: CGRID( :,:,:,: )
            INTEGER, INTENT( OUT )    :: STDATE, STTIME
            INTEGER, INTENT( OUT )    :: TSTEP( 2 )
            INTEGER, INTENT( OUT )    :: NSTEPS
         END SUBROUTINE INITSCEN
         SUBROUTINE PA_INIT ( CGRID, JDATE, JTIME, TSTEP )
            IMPLICIT NONE
            REAL, POINTER             :: CGRID( :,:,:,: )
            INTEGER, INTENT( IN )     :: JDATE, JTIME
            INTEGER, INTENT( IN )     :: TSTEP( 2 )
         END SUBROUTINE PA_INIT
         SUBROUTINE SUBDRIVER_FWD(CGRID, STDATE, STTIME, TSTEP, NSTEPS)
            IMPLICIT NONE
            REAL, POINTER             :: CGRID( :,:,:,: )
            INTEGER                   :: STDATE, STTIME 
            INTEGER                   :: TSTEP( 2 ), NSTEPS            
         END SUBROUTINE SUBDRIVER_FWD
         SUBROUTINE FDDRIVER_BWD(CGRID, f, LGRID, STDATE, STTIME, TSTEP, NSTEPS)
            IMPLICIT NONE
            REAL, POINTER             :: CGRID( :,:,:,: )
            REAL, POINTER             :: LGRID( :,:,:,: )
            DOUBLE PRECISION          :: f         
            INTEGER                   :: STDATE, STTIME 
            INTEGER                   :: TSTEP( 2 ), NSTEPS            
         END SUBROUTINE FDDRIVER_BWD
         SUBROUTINE RD_CHK ( CGRID, JDATE, JTIME )
            IMPLICIT NONE
            REAL, POINTER             :: CGRID( :,:,:,: )
            INTEGER, INTENT( IN )     :: JDATE, JTIME
            END SUBROUTINE RD_CHK 
      END INTERFACE


C-----------------------------------------------------------------------

C Get number of species, and starting indices for CGRID array.

      CALL CGRID_MAP ( NSPCSD, GC_STRT, AE_STRT, NR_STRT, TR_STRT )
      
C Start up processor communication and define horizontal domain decomposition
C and vertical layer structure

      CALL PAR_INIT( PNAME, NSPCSD, CLOCK, PAR_ERR )

      IF ( PAR_ERR .NE. 0 ) THEN
         XMSG = 'Error in PAR_INIT'
         CALL M3EXIT ( PNAME, 0, 0, XMSG, XSTAT2 )
         END IF

      LOGDEV = INIT3 ()

C Initialize PCGRID

      IF ( .NOT. PCGRID_INIT() ) THEN
         XMSG = 'Failure defining horizontal domain'
         CALL M3EXIT ( PNAME, 0, 0, XMSG, XSTAT2  )
         END IF
      
      CGRID => PCGRID(1:MY_NCOLS,1:MY_NROWS,:,:)   ! required for PinG

C Initialize conc field: Copy IC's to CONC file as step 0
C Convention: the input file concentration units are always ppmV.

      CALL INITSCEN ( CGRID, STDATE, STTIME, TSTEP, NSTEPS )

C Verify input file header consistency with COORD.EXT and run duration

      CALL FLCHECK ( STDATE, STTIME, TSTEP( 1 ) )

      JDATE = STDATE
      JTIME = STTIME

      IF ( LIPR .OR. LIRR ) CALL PA_INIT ( CGRID, STDATE, STTIME, TSTEP )

c     initialize dynamic arrays

      ALLOCATE ( CGRIDREF( MY_NCOLS,MY_NROWS,NLAYS,NSPCSD ), STAT = ALLOCSTAT )
      IF ( ALLOCSTAT .NE. 0 ) THEN
         XMSG = 'CGRIDREF memory allocation failed'
         CALL M3EXIT ( PNAME, JDATE, JTIME, XMSG, XSTAT1 )
      END IF     
      
      ALLOCATE ( FIRSTPERT( MY_NCOLS,MY_NROWS,NLAYS,NSPCSD ), STAT = ALLOCSTAT )
      IF ( ALLOCSTAT .NE. 0 ) THEN
         XMSG = 'FIRSTPERT memory allocation failed'
         CALL M3EXIT ( PNAME, JDATE, JTIME, XMSG, XSTAT1 )
      END IF     

      ALLOCATE ( LGRID( MY_NCOLS,MY_NROWS,NLAYS,NSPCSD ), STAT = ALLOCSTAT )
      IF ( ALLOCSTAT .NE. 0 ) THEN
         XMSG = 'LGRID memory allocation failed'
         CALL M3EXIT ( PNAME, JDATE, JTIME, XMSG, XSTAT1 )
      END IF

c     open checkpoint files
     
      IF ( .NOT. OPEN3( CONC_L2CHK, FSRDWR3, PNAME ) ) THEN
         XMSG = 'Could not open ' // CONC_L2CHK // ' file for update'
      END IF

      IF ( .NOT. OPEN3( CONC_L4CHK, FSRDWR3, PNAME ) ) THEN
         XMSG = 'Could not open ' // CONC_L4CHK // ' file for update'
      END IF

      IF ( .NOT. OPEN3( CONC_L3CHK, FSRDWR3, PNAME ) ) THEN
         XMSG = 'Could not open ' // CONC_L3CHK // ' file for update'
      END IF

      IF ( .NOT. OPEN3( CONC_CHK, FSRDWR3, PNAME ) ) THEN
         XMSG = 'Could not open ' // CONC_CHK // ' file for update'
      END IF

      IF ( .NOT. OPEN3( ADJFAC_CHK, FSRDWR3, PNAME ) ) THEN
         XMSG = 'Could not open ' // CONC_CHK // ' file for update'
      END IF

      IF ( .NOT. OPEN3( CONC_XADVCHK, FSRDWR3, PNAME ) ) THEN
         XMSG = 'Could not open ' // CONC_CHK // ' file for update'
      END IF
      
      IF ( .NOT. OPEN3( CONC_YADVCHK, FSRDWR3, PNAME ) ) THEN
         XMSG = 'Could not open ' // CONC_CHK // ' file for update'
      END IF
      

      
C     =================================================================
C     FINITE DIFFERENCE TEST INITIALIZATIONS
C     FIRST  RUN - introduce first perturbation 
C     SECOND RUN - introduce second perturbation
C     SPC = 4 corresponds to OZONE in CGRID(:,:,:,SPC)  
C     =================================================================
      
      CGRIDREF = CGRID
      
      SPB = 4
      SPE = 4
      
C     lay down observations
      CALL SUBDRIVER_FWD(CGRID, STDATE, STTIME, TSTEP, NSTEPS)
      
      CGRID = CGRIDREF
      CGRID(:,:,:,SPB:SPE) = CGRID(:,:,:,SPB:SPE)*(1.1+9.0*eps)

      FIRSTPERT = CGRID
      
C     forward & adjoint model run: calculate cost function (f1)
      CALL FDDRIVER_BWD(CGRID, f, LGRID, STDATE, STTIME, TSTEP, NSTEPS)
      
      PRINT*,'==========================='
      PRINT*,'Objective Function = ',f
      PRINT*,'==========================='
      
      f1 = f
      
      CGRID = CGRIDREF
      CGRID(:,:,:,SPB:SPE) = CGRID(:,:,:,SPB:SPE)*(1.1+10.0*eps)
      
C     forward & adjoint model run: calculate cost function (f2)
      CALL FDDRIVER_BWD(CGRID, f, LGRID, STDATE, STTIME, TSTEP, NSTEPS)      

      PRINT*,'==========================='
      PRINT*,'Objective Function = ',f
      PRINT*,'==========================='
      
C     =============================================
C     COMPARE LHS: COST_FUNC1-COST_FUNC2 vs 
C     RHS: (CGRID,LGRID) dot product
C     =============================================
 
      PRINT*,'---------------------------------------'
      PRINT*,'***************************************'
      PRINT*,'LHS: costfunc1 - costfunc2  = ',f-f1   ! costfunc2 - costfunc1
      
      JDATE = STDATE
      JTIME = STTIME
      
      CALL RD_CHK(CGRID,JDATE,JTIME)
      
      sum=0.0 
      DO m=1,MY_NCOLS
         DO n=1,MY_NROWS
            DO o=1,NLAYS
               DO p=1,NSPCSD-1
                  sum = sum + (CGRID(m,n,o,p)-FIRSTPERT(m,n,o,p))
     &                 *LGRID(m,n,o,p)
               END DO
            END DO
         END DO
      END DO
      PRINT*,'RHS: LGRID.(CGRID1-CGRID2)  = ',sum
      relerr = (sum-f+f1)/sum
      PRINT*,'relative difference = ',relerr,' ( ',relerr*100,'% )'
      PRINT*,'***************************************'
      PRINT*,'---------------------------------------'	 	 
                  
c     Close checkpoint files

      IF ( .NOT. CLOSE3( CONC_L2CHK ) ) THEN
         XMSG = 'Could not close ' // CONC_L2CHK
         CALL M3WARN( PNAME, STDATE, STTIME, XMSG )
      END IF

      IF ( .NOT. CLOSE3( CONC_L4CHK ) ) THEN
         XMSG = 'Could not close ' // CONC_L4CHK
         CALL M3WARN( PNAME, STDATE, STTIME, XMSG )
      END IF

      IF ( .NOT. CLOSE3( CONC_L3CHK ) ) THEN
         XMSG = 'Could not close ' // CONC_L3CHK
         CALL M3WARN( PNAME, STDATE, STTIME, XMSG )
      END IF

      IF ( .NOT. CLOSE3( CONC_CHK ) ) THEN
         XMSG = 'Could not close ' // CONC_CHK
         CALL M3WARN( PNAME, STDATE, STTIME, XMSG )
      END IF
      
      IF ( .NOT. CLOSE3( ADJFAC_CHK ) ) THEN
         XMSG = 'Could not close ' // ADJFAC_CHK
         CALL M3WARN( PNAME, STDATE, STTIME, XMSG )
      END IF

      IF ( .NOT. CLOSE3( CONC_XADVCHK ) ) THEN
         XMSG = 'Could not close ' //CONC_XADVCHK
         CALL M3WARN( PNAME, STDATE, STTIME, XMSG )
      END IF

      IF ( .NOT. CLOSE3( CONC_YADVCHK ) ) THEN
         XMSG = 'Could not close ' //CONC_YADVCHK
         CALL M3WARN( PNAME, STDATE, STTIME, XMSG )
      END IF
      
C     Clean up communications.
      
c$$$      CALL PAR_TERM ( PNAME, JDATE, JTIME, CLOCK )
      
      END
