   
C***********************************************************************
C   Portions of Models-3/CMAQ software were developed or based on      *
C   information from various groups: Federal Government employees,     *
C   contractors working on a United States Government contract, and    *
C   non-Federal sources (including research institutions).  These      *
C   research institutions have given the Government permission to      *
C   use, prepare derivative works, and distribute copies of their      *
C   work in Models-3/CMAQ to the public and to permit others to do     *
C   so.  EPA therefore grants similar permissions for use of the       *
C   Models-3/CMAQ software, but users are requested to provide copies  *
C   of derivative works to the Government without restrictions as to   *
C   use by others.  Users are responsible for acquiring their own      *
C   copies of commercial software associated with Models-3/CMAQ and    *
C   for complying with vendor requirements.  Software copyrights by    *
C   the MCNC Environmental Modeling Center are used with their         *
C   permissions subject to the above restrictions.                     *
C***********************************************************************


C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
      SUBROUTINE FDDRIVER_BWD(CGRID, f, LGRID, STDATE, STTIME, TSTEP, NSTEPS)

C-----------------------------------------------------------------------
C Function:
C     Subroutine to perform one forward and one backward run to calculate the
C     cost function and update LGRID for observation misfit only.

C CHECKPOINT FILES:
C     CONC_L3CHK = Reference/Base checkpoint file
C     CONC_CHK   = Current concentration
 
C INPUT:
C     CGRID

C OUTPUT:
C     f,LGRID

C Subroutines and functions called:
C     INITSCEN, ADVSTEP, M3EXIT, WRITE3
C     science processes SCIPROC, SCIPROC_CADJ, PA_OUTPUT
C     OBS_GRAD_UPDATE -> calculate cost function & update LGRID
C     Read and write checkpoint files -> WR_CHK,RD_CHK
 
C Revision History:
C     Kumaresh Singh & Sandu,A. Apr 07 - Added to perform forward & adjoint 
C     model run for Finite-Difference calculations
C-----------------------------------------------------------------------

      USE PCGRID_DEFN           ! inherits HGRD_DEFN and CGRID_SPCS
      USE VGRD_DEFN             ! vertical layer specifications
      USE AVG_CONC              ! integral average CONC
      USE XFIRSTM               ! contain xfirst variable

      IMPLICIT NONE

C Include Files:

!     INCLUDE SUBST_VGRD_ID     ! vertical dimensioning parameters
      INCLUDE SUBST_IOPARMS     ! I/O parameters definitions
      INCLUDE SUBST_IODECL      ! I/O definitions and declarations
      INCLUDE SUBST_FILES_ID    ! I/O definitions and declarations
      INCLUDE SUBST_PACTL_ID    ! PA control parameters

C External Functions (not already declared by IODECL3.EXT):

      LOGICAL, EXTERNAL :: ENVYN ! get environment variable as boolean

C Local variables:

      INTEGER      LOGDEV       ! FORTRAN unit number for log file
      INTEGER      STDATE       ! starting date,    format YYYYDDD
      INTEGER      STTIME       ! starting time,    format HHMMSS
      INTEGER      TSTEP( 2 )   ! time step vector (HHMMSS)
                                ! TSTEP(1) = local output step
                                ! TSTEP(2) = sciproc sync. step (chem)
!     INTEGER      ASTEP( NLAYS )
      INTEGER      SDATE, STIME
      INTEGER, ALLOCATABLE :: ASTEP( : )
      INTEGER      NSTEPS       ! run duration: number of output time steps
      INTEGER      NREPS        ! number of model time steps per output step
      INTEGER      ISTEP        ! current output time step number
      INTEGER      IREP         ! model step number within this output step
      INTEGER      JDATE        ! current model date, coded YYYYDDD
      INTEGER      JTIME        ! current model time, coded HHMMSS
      INTEGER      C, R, L, K, S, V     ! loop induction variables
      INTEGER      ALLOCSTAT
      INTEGER      SAVETIME,SAVEDATE
      INTEGER      RESDATE, RESTIME

      CHARACTER( 16 ) :: PNAME = 'FDDRIVER_BWD'
      CHARACTER( 96 ) :: XMSG = ' '

      REAL, POINTER :: CGRID( :,:,:,: )
      REAL, POINTER :: CGRIDREF( :,:,:,: )
c-----------------------------------------
      REAL, POINTER :: LGRID( :,:,:,: )

      DOUBLE PRECISION  f
      REAL  D,relerr,CF
 
      INTEGER  m,n,o,p,istatus,I

      LOGICAL, SAVE :: FIRSTIME = .TRUE.
      REAL,    SAVE :: TEMP
c------------------------------------------

      REAL, ALLOCATABLE :: AGRID( :,:,:,: )
      REAL    DIVFAC      ! trapezoidal average factor
      INTEGER A_NLYS

      INTEGER PAR_ERR     ! Error code from parallel initialization. 
      REAL    CLOCK       ! Wall-clock time (sec) at initialization.

      INTERFACE
         SUBROUTINE INITSCEN 
     &                      ( CGRID, STDATE, STTIME, TSTEP, NSTEPS )
            IMPLICIT NONE
            REAL, POINTER             :: CGRID( :,:,:,: )
            INTEGER, INTENT( OUT )    :: STDATE, STTIME
            INTEGER, INTENT( OUT )    :: TSTEP( 2 )
            INTEGER, INTENT( OUT )    :: NSTEPS
         END SUBROUTINE INITSCEN
         SUBROUTINE ADVSTEP ( JDATE, JTIME, TSTEP, ASTEP, NREPS )
            IMPLICIT NONE
            INTEGER, INTENT( IN )     :: JDATE, JTIME
            INTEGER, INTENT( IN )     :: TSTEP( 2 )
            INTEGER, INTENT( IN )     :: ASTEP( : )
            INTEGER, INTENT( OUT )    :: NREPS
         END SUBROUTINE ADVSTEP
         SUBROUTINE CKSUMMER ( PNAME, CGRID, JDATE, JTIME )
            IMPLICIT NONE
            CHARACTER( * ), INTENT( IN ) :: PNAME
            REAL, POINTER             :: CGRID( :,:,:,: )
            INTEGER, INTENT( IN )     :: JDATE, JTIME
         END SUBROUTINE CKSUMMER
         SUBROUTINE PA_INIT ( CGRID, JDATE, JTIME, TSTEP )
            IMPLICIT NONE
            REAL, POINTER             :: CGRID( :,:,:,: )
            INTEGER, INTENT( IN )     :: JDATE, JTIME
            INTEGER, INTENT( IN )     :: TSTEP( 2 )
         END SUBROUTINE PA_INIT
         SUBROUTINE SCIPROC ( CGRID, JDATE, JTIME, TSTEP, ASTEP )
            IMPLICIT NONE
!           INCLUDE SUBST_VGRD_ID     ! vertical dimensioning parameters
            REAL, POINTER             :: CGRID( :,:,:,: )
            INTEGER, INTENT( IN OUT ) :: JDATE, JTIME
            INTEGER, INTENT( IN )     :: TSTEP( 2 )
            INTEGER, INTENT( IN )     :: ASTEP( : )
         END SUBROUTINE SCIPROC
         SUBROUTINE SCIPROC_CADJ ( LGRID, CGRID, JDATE, JTIME, TSTEP, ASTEP)
            IMPLICIT NONE
            REAL, POINTER             :: LGRID(:,:,:,:), CGRID(:,:,:,:)
            INTEGER, INTENT( IN OUT ) :: JDATE, JTIME
            INTEGER, INTENT( IN )     :: TSTEP( 2 ), ASTEP( : )
         END SUBROUTINE SCIPROC_CADJ
          SUBROUTINE WR_CHK ( CGRID, JDATE, JTIME )
            IMPLICIT NONE
            REAL, POINTER             :: CGRID( :,:,:,: )
            INTEGER, INTENT( IN )     :: JDATE, JTIME
         END SUBROUTINE WR_CHK
         SUBROUTINE RD_CHK ( CGRID, JDATE, JTIME )
            IMPLICIT NONE
            REAL, POINTER             :: CGRID( :,:,:,: )
            INTEGER, INTENT( IN )     :: JDATE, JTIME
         END SUBROUTINE RD_CHK
         SUBROUTINE OBS_GRAD_UPDATE ( CGRID, LGRID, CF, JDATE, JTIME )
            IMPLICIT NONE
            REAL, POINTER             :: CGRID( :,:,:,: )
            REAL, POINTER             :: LGRID( :,:,:,: )
            REAL                      :: CF
            INTEGER, INTENT( IN )     :: JDATE, JTIME
         END SUBROUTINE OBS_GRAD_UPDATE
         SUBROUTINE WR_CONC ( CGRID, JDATE, JTIME )
            IMPLICIT NONE
            REAL, POINTER             :: CGRID( :,:,:,: )
            INTEGER, INTENT( IN )     :: JDATE, JTIME
         END SUBROUTINE WR_CONC
         SUBROUTINE WR_ACONC ( AGRID, JDATE, JTIME, TSTEP )
            IMPLICIT NONE
            REAL,    INTENT( IN )     :: AGRID( :,:,:,: )
            INTEGER, INTENT( IN OUT ) :: JDATE, JTIME
            INTEGER, INTENT( IN )     :: TSTEP
         END SUBROUTINE WR_ACONC
         SUBROUTINE PA_OUTPUT ( CGRID, JDATE, JTIME )
            IMPLICIT NONE
            REAL, POINTER             :: CGRID( :,:,:,: )
            INTEGER, INTENT( IN )     :: JDATE, JTIME
         END SUBROUTINE PA_OUTPUT
      END INTERFACE


C-----------------------------------------------------------------------

C Get number of species, and starting indices for CGRID array.

      CALL CGRID_MAP ( NSPCSD, GC_STRT, AE_STRT, NR_STRT, TR_STRT )
      
C Start up processor communication and define horizontal domain decomposition
C and vertical layer structure


      JDATE = STDATE
      JTIME = STTIME

      ALLOCATE ( AGRID( MY_NCOLS,MY_NROWS,A_NLYS,N_ASPCS ), STAT = ALLOCSTAT )
      IF ( ALLOCSTAT .NE. 0 ) THEN
         XMSG = 'AGRID memory allocation failed'
         CALL M3EXIT ( PNAME, JDATE, JTIME, XMSG, XSTAT1 )
         END IF

      AGRID = 0.0

      ALLOCATE ( ASTEP( NLAYS ), STAT = ALLOCSTAT )
      IF ( ALLOCSTAT .NE. 0 ) THEN
         XMSG = 'ASTEP memory allocation failed'
         CALL M3EXIT ( PNAME, JDATE, JTIME, XMSG, XSTAT1 )
      END IF
   
C     Initialize XFIRST for the Forward simulation
      XFIRST = .TRUE.

C     science process sequence cgrid,:

c     write CGRID to CONC_CHK for initial time step
      CALL WR_CHK ( CGRID, JDATE, JTIME )
         
C     Main processing loop:
      
      DO 201 ISTEP = 1, NSTEPS  ! output time step loop
         
C     Get synchronization and advection time steps, TSTEP(2), ASTEP(L) and NREPS
         
         CALL ADVSTEP ( JDATE, JTIME, TSTEP, ASTEP, NREPS )
         
         DO V = 1, N_ASPCS
            S = AVG_CONC_MAP( V )
            L = 0
            DO K = ACONC_BLEV, ACONC_ELEV
               L = L + 1
               DO R = 1, MY_NROWS
                  DO C = 1, MY_NCOLS
                     AGRID( C,R,L,V ) = CGRID( C,R,K,S )
                  END DO
               END DO
            END DO
         END DO
         
C     science process sequence:
         
         DO IREP = 1, NREPS - 1
            
            CALL SCIPROC ( CGRID, JDATE, JTIME, TSTEP, ASTEP )

c           write CGRID values to CONC_CHK for current JDATE,JTIME
            CALL WR_CHK ( CGRID, JDATE, JTIME )

            DO V = 1, N_ASPCS
               S = AVG_CONC_MAP( V )
               L = 0
               DO K = ACONC_BLEV, ACONC_ELEV
                  L = L + 1
                  DO R = 1, MY_NROWS
                     DO C = 1, MY_NCOLS
                        AGRID( C,R,L,V ) = AGRID( C,R,L,V )
     &                       + 2.0 * CGRID( C,R,K,S )
                     END DO
                  END DO
               END DO
            END DO
            
         END DO
         
         CALL SCIPROC ( CGRID, JDATE, JTIME, TSTEP, ASTEP )
        
c        write CGRID values to CONC_CHK for final time step
         CALL WR_CHK ( CGRID, JDATE, JTIME )

         DIVFAC = 0.5 / FLOAT( NREPS )

         DO V = 1, N_ASPCS
            S = AVG_CONC_MAP( V )
            L = 0
            DO K = ACONC_BLEV, ACONC_ELEV
               L = L + 1
               DO R = 1, MY_NROWS
                  DO C = 1, MY_NCOLS
                     AGRID( C,R,L,V ) = DIVFAC * ( AGRID( C,R,L,V )
     &                    +            CGRID( C,R,K,S ) )
                  END DO
               END DO
            END DO
         END DO

C     write conc fields

         CALL WR_CONC ( CGRID, JDATE, JTIME )

         CALL WR_ACONC ( AGRID, JDATE, JTIME, TSTEP( 1 ) )

         IF ( LIPR .OR. LIRR ) CALL PA_OUTPUT ( CGRID, JDATE, JTIME )

 201  CONTINUE

c---------------------------
c     Backward in Time Loop 
c---------------------------
      
      PRINT*,'--------------------------'
      PRINT*,'Backward loop starts here'
      PRINT*,'--------------------------'

      LGRID = 0.000000

      f = 0.d0

C     Read CGRID for final time step
      CALL RD_CHK ( CGRID, JDATE, JTIME )
      
      RESDATE = JDATE
      RESTIME = JTIME

C     ============================================
C     OBSERVATION FORCING UPDATE
C     ============================================
      CALL OBS_GRAD_UPDATE( CGRID, LGRID, CF, JDATE, JTIME )
      f = f + CF

C     Main processing loop:

      DO 202 ISTEP = NSTEPS, 1, -1 ! output time step loop
C     Get synchronization and advection time steps, TSTEP(2), ASTEP(L) and NREPS
         
         SDATE = JDATE
         STIME = JTIME
         CALL NEXTIME(SDATE, STIME, -10000)
         
         CALL ADVSTEP ( SDATE, STIME, TSTEP, ASTEP, NREPS )
         
C     Reverse science process sequence:
         
         DO IREP = NREPS, 1, -1
            
            CALL NEXTIME ( RESDATE, RESTIME,-TSTEP( 2 ) )
            
C           Read CGRID for current time step
            CALL RD_CHK ( CGRID, RESDATE, RESTIME )

C           perform adjoint science process calculations on LGRID
            CALL SCIPROC_CADJ(LGRID, CGRID, JDATE, JTIME, TSTEP, ASTEP)  

C           ============================================
C           OBSERVATION FORCING UPDATE
C           ============================================
            CALL OBS_GRAD_UPDATE( CGRID, LGRID, CF, JDATE, JTIME ) 
            f = f + CF

         END DO
         
 202  CONTINUE


      DEALLOCATE ( AGRID, STAT = ALLOCSTAT )
      IF ( ALLOCSTAT .NE. 0 ) THEN
         XMSG = 'AGRID memory deallocation failed'
         CALL M3EXIT ( PNAME, JDATE, JTIME, XMSG, XSTAT1 )
      END IF

      DEALLOCATE ( ASTEP, STAT = ALLOCSTAT )
      IF ( ALLOCSTAT .NE. 0 ) THEN
         XMSG = 'ASTEP memory deallocation failed'
         CALL M3EXIT ( PNAME, JDATE, JTIME, XMSG, XSTAT1 )
      END IF
      
      
C     Clean up communications.
      
c$$$      CALL PAR_TERM ( PNAME, JDATE, JTIME, CLOCK )
      
      END SUBROUTINE
