      
C***********************************************************************
C   Portions of Models-3/CMAQ software were developed or based on      *
C   information from various groups: Federal Government employees,     *
C   contractors working on a United States Government contract, and    *
C   non-Federal sources (including research institutions).  These      *
C   research institutions have given the Government permission to      *
C   use, prepare derivative works, and distribute copies of their      *
C   work in Models-3/CMAQ to the public and to permit others to do     *
C   so.  EPA therefore grants similar permissions for use of the       *
C   Models-3/CMAQ software, but users are requested to provide copies  *
C   of derivative works to the Government without restrictions as to   *
C   use by others.  Users are responsible for acquiring their own      *
C   copies of commercial software associated with Models-3/CMAQ and    *
C   for complying with vendor requirements.  Software copyrights by    *
C   the MCNC Environmental Modeling Center are used with their         *
C   permissions subject to the above restrictions.                     *
C***********************************************************************

C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
      SUBROUTINE STDRIVER_BWD(CGRID, LGRID, STDATE, STTIME, TSTEP, NSTEPS)
C-----------------------------------------------------------------------
C Function:
C     Subroutine to perform one forward and one backward run to generate
C     adjoint trajectory.(with discrete adjoint for advection)

C CHECKPOINT FILES:
C     CONC_CHK   = CGRID concentration
C     CONC_L5CHK = LGRID values
C     CONC_L6CHK = EMGRID (emission grid) values

C Subroutines and functions called:
C     INITSCEN, ADVSTEP, M3EXIT, WRITE3
C     science processes  -> SCIPROC, SCIPROC_DADJ, PA_OUTPUT
C     Initializing LGRID -> DEFINE_RECEPTOR       
C     Read and write checkpoint files -> WR_CHK,RD_CHK,WR_L5CHK
 
C Revision History:
C     Kumaresh Singh & Sandu,A. Apr 07 - Added to perform forward and 
C     adjoint model run for Sensitivity calculations
C     Tianyi Gou & Sandu,A.            - USE XFIRSTM and initilize xfirst
C-----------------------------------------------------------------------

      USE PCGRID_DEFN           ! inherits HGRD_DEFN and CGRID_SPCS
      USE VGRD_DEFN             ! vertical layer specifications
      USE AVG_CONC              ! integral average CONC
      USE XFIRSTM               ! contain xfirst variable


      IMPLICIT NONE

C Include Files:

!     INCLUDE SUBST_VGRD_ID     ! vertical dimensioning parameters
      INCLUDE SUBST_IOPARMS     ! I/O parameters definitions
      INCLUDE SUBST_IODECL      ! I/O definitions and declarations
      INCLUDE SUBST_FILES_ID    ! I/O definitions and declarations
      INCLUDE SUBST_PACTL_ID    ! PA control parameters
      INCLUDE SUBST_GC_DIFF   ! gas chem diffusion species and map table
      INCLUDE SUBST_AE_DIFF   ! aerosol diffusion species and map table
      INCLUDE SUBST_NR_DIFF   ! non-react diffusion species and map table
      INCLUDE SUBST_TR_DIFF   ! tracer diffusion species and map table

C External Functions (not already declared by IODECL3.EXT):

      LOGICAL, EXTERNAL :: ENVYN ! get environment variable as boolean

C Local variables:

      INTEGER      LOGDEV       ! FORTRAN unit number for log file
      INTEGER      STDATE       ! starting date,    format YYYYDDD
      INTEGER      STTIME       ! starting time,    format HHMMSS
      INTEGER      TSTEP( 2 )   ! time step vector (HHMMSS)
                                ! TSTEP(1) = local output step
                                ! TSTEP(2) = sciproc sync. step (chem)
!     INTEGER      ASTEP( NLAYS )
      INTEGER      SDATE, STIME
      INTEGER, ALLOCATABLE :: ASTEP( : )
      INTEGER      NSTEPS       ! run duration: number of output time steps
      INTEGER      NREPS        ! number of model time steps per output step
      INTEGER      ISTEP        ! current output time step number
      INTEGER      IREP         ! model step number within this output step
      INTEGER      JDATE        ! current model date, coded YYYYDDD
      INTEGER      JTIME        ! current model time, coded HHMMSS
      INTEGER      C, R, L, K, S, V     ! loop induction variables
      INTEGER      ALLOCSTAT
      INTEGER      SAVETIME,SAVEDATE
      INTEGER      RESDATE, RESTIME
      INTEGER      COL,ROW,LAY    ! add on 10/13/2008

      CHARACTER( 16 ) :: PNAME = 'STDRIVER_BWD'
      CHARACTER( 96 ) :: XMSG = ' '

      REAL, POINTER :: CGRID( :,:,:,: )
c-----------------------------------------
      REAL, POINTER :: LGRID( :,:,:,: )
      REAL, POINTER :: EMGRID( :,:,:,: )

C global diffusion species
      INTEGER, PARAMETER :: N_SPC_DIFF = N_GC_DIFF
     &                                 + N_AE_DIFF
     &                                 + N_NR_DIFF
     &                                 + N_TR_DIFF

      REAL  D,relerr,CF
 
      INTEGER  m,n,o,p,istatus,I

      LOGICAL, SAVE :: LASTDAY = .TRUE.
      REAL,    SAVE :: TEMP
c------------------------------------------

      REAL, ALLOCATABLE :: AGRID( :,:,:,: )
      REAL    DIVFAC      ! trapezoidal average factor
      INTEGER A_NLYS

      INTEGER PAR_ERR     ! Error code from parallel initialization. 
      REAL    CLOCK       ! Wall-clock time (sec) at initialization.

      INTERFACE
         SUBROUTINE INITSCEN 
     &                      ( CGRID, STDATE, STTIME, TSTEP, NSTEPS )
            IMPLICIT NONE
            REAL, POINTER             :: CGRID( :,:,:,: )
            INTEGER, INTENT( OUT )    :: STDATE, STTIME
            INTEGER, INTENT( OUT )    :: TSTEP( 2 )
            INTEGER, INTENT( OUT )    :: NSTEPS
         END SUBROUTINE INITSCEN
         SUBROUTINE ADVSTEP ( JDATE, JTIME, TSTEP, ASTEP, NREPS )
            IMPLICIT NONE
            INTEGER, INTENT( IN )     :: JDATE, JTIME
            INTEGER, INTENT( IN )     :: TSTEP( 2 )
            INTEGER, INTENT( IN )     :: ASTEP( : )
            INTEGER, INTENT( OUT )    :: NREPS
         END SUBROUTINE ADVSTEP
         SUBROUTINE CKSUMMER ( PNAME, CGRID, JDATE, JTIME )
            IMPLICIT NONE
            CHARACTER( * ), INTENT( IN ) :: PNAME
            REAL, POINTER             :: CGRID( :,:,:,: )
            INTEGER, INTENT( IN )     :: JDATE, JTIME
         END SUBROUTINE CKSUMMER
         SUBROUTINE PA_INIT ( CGRID, JDATE, JTIME, TSTEP )
            IMPLICIT NONE
            REAL, POINTER             :: CGRID( :,:,:,: )
            INTEGER, INTENT( IN )     :: JDATE, JTIME
            INTEGER, INTENT( IN )     :: TSTEP( 2 )
         END SUBROUTINE PA_INIT
         SUBROUTINE SCIPROC ( CGRID, JDATE, JTIME, TSTEP, ASTEP )
            IMPLICIT NONE
!           INCLUDE SUBST_VGRD_ID     ! vertical dimensioning parameters
            REAL, POINTER             :: CGRID( :,:,:,: )
            INTEGER, INTENT( IN OUT ) :: JDATE, JTIME
            INTEGER, INTENT( IN )     :: TSTEP( 2 )
            INTEGER, INTENT( IN )     :: ASTEP( : )
         END SUBROUTINE SCIPROC
         SUBROUTINE SCIPROC_DADJ ( LGRID, EMGRID, JDATE, JTIME, TSTEP, ASTEP)
            IMPLICIT NONE
            REAL, POINTER             :: LGRID(:,:,:,:), EMGRID(:,:,:,:)
            INTEGER, INTENT( IN OUT ) :: JDATE, JTIME
            INTEGER, INTENT( IN )     :: TSTEP( 2 ), ASTEP( : )
         END SUBROUTINE SCIPROC_DADJ
          SUBROUTINE WR_CHK ( CGRID, JDATE, JTIME )
            IMPLICIT NONE
            REAL, POINTER             :: CGRID( :,:,:,: )
            INTEGER, INTENT( IN )     :: JDATE, JTIME
         END SUBROUTINE WR_CHK
          SUBROUTINE WR_L5CHK ( CGRID, JDATE, JTIME )
            IMPLICIT NONE
            REAL, POINTER             :: CGRID( :,:,:,: )
            INTEGER, INTENT( IN )     :: JDATE, JTIME
         END SUBROUTINE WR_L5CHK
          SUBROUTINE RD_L5CHK ( CGRID, JDATE, JTIME )
            IMPLICIT NONE
            REAL, POINTER             :: CGRID( :,:,:,: )
            INTEGER, INTENT( IN )     :: JDATE, JTIME
         END SUBROUTINE RD_L5CHK
         SUBROUTINE RD_CHK ( CGRID, JDATE, JTIME )
            IMPLICIT NONE
            REAL, POINTER             :: CGRID( :,:,:,: )
            INTEGER, INTENT( IN )     :: JDATE, JTIME
         END SUBROUTINE RD_CHK
         SUBROUTINE DEFINE_RECEPTOR ( LGRID )
            IMPLICIT NONE
            REAL, POINTER             :: LGRID( :,:,:,: )
         END SUBROUTINE DEFINE_RECEPTOR
         SUBROUTINE WR_CONC ( CGRID, JDATE, JTIME )
            IMPLICIT NONE
            REAL, POINTER             :: CGRID( :,:,:,: )
            INTEGER, INTENT( IN )     :: JDATE, JTIME
         END SUBROUTINE WR_CONC
         SUBROUTINE WR_ACONC ( AGRID, JDATE, JTIME, TSTEP )
            IMPLICIT NONE
            REAL,    INTENT( IN )     :: AGRID( :,:,:,: )
            INTEGER, INTENT( IN OUT ) :: JDATE, JTIME
            INTEGER, INTENT( IN )     :: TSTEP
         END SUBROUTINE WR_ACONC
         SUBROUTINE PA_OUTPUT ( CGRID, JDATE, JTIME )
            IMPLICIT NONE
            REAL, POINTER             :: CGRID( :,:,:,: )
            INTEGER, INTENT( IN )     :: JDATE, JTIME
         END SUBROUTINE PA_OUTPUT
      END INTERFACE


C-----------------------------------------------------------------------

C Get number of species, and starting indices for CGRID array.

      CALL CGRID_MAP ( NSPCSD, GC_STRT, AE_STRT, NR_STRT, TR_STRT )
      
C Start up processor communication and define horizontal domain decomposition
C and vertical layer structure

      JDATE = STDATE
      JTIME = STTIME

      ALLOCATE ( AGRID( MY_NCOLS,MY_NROWS,A_NLYS,N_ASPCS ), STAT = ALLOCSTAT )
      IF ( ALLOCSTAT .NE. 0 ) THEN
         XMSG = 'AGRID memory allocation failed'
         CALL M3EXIT ( PNAME, JDATE, JTIME, XMSG, XSTAT1 )
         END IF

      ALLOCATE ( EMGRID( MY_NCOLS,MY_NROWS,NLAYS,N_SPC_DIFF ), STAT = ALLOCSTAT )
      IF ( ALLOCSTAT .NE. 0 ) THEN
         XMSG = 'EMGRID memory allocation failed'
         CALL M3EXIT ( PNAME, JDATE, JTIME, XMSG, XSTAT1 )
      END IF

      AGRID    = 0.0

      ALLOCATE ( ASTEP( NLAYS ), STAT = ALLOCSTAT )
      IF ( ALLOCSTAT .NE. 0 ) THEN
         XMSG = 'ASTEP memory allocation failed'
         CALL M3EXIT ( PNAME, JDATE, JTIME, XMSG, XSTAT1 )
      END IF
      
C     Initialize XFIRST for the Forward simulation
      XFIRST = .TRUE.

C     science process sequence cgrid,:

c     write CGRID to CONC_CHK for initial time step
      CALL WR_CHK ( CGRID, JDATE, JTIME )
         
C     Main processing loop:
      
      DO 201 ISTEP = 1, NSTEPS  ! output time step loop
         
C     Get synchronization and advection time steps, TSTEP(2), ASTEP(L) and NREPS
         
         CALL ADVSTEP ( JDATE, JTIME, TSTEP, ASTEP, NREPS )
         
         DO V = 1, N_ASPCS
            S = AVG_CONC_MAP( V )
            L = 0
            DO K = ACONC_BLEV, ACONC_ELEV
               L = L + 1
               DO R = 1, MY_NROWS
                  DO C = 1, MY_NCOLS
                     AGRID( C,R,L,V ) = CGRID( C,R,K,S )
                  END DO
               END DO
            END DO
         END DO
         
C     science process sequence:
         
         DO IREP = 1, NREPS - 1
            
            CALL SCIPROC ( CGRID, JDATE, JTIME, TSTEP, ASTEP )

c           write CGRID to CONC_CHK for dynamic time step
            CALL WR_CHK ( CGRID, JDATE, JTIME )
            
            DO V = 1, N_ASPCS
               S = AVG_CONC_MAP( V )
               L = 0
               DO K = ACONC_BLEV, ACONC_ELEV
                  L = L + 1
                  DO R = 1, MY_NROWS
                     DO C = 1, MY_NCOLS
                        AGRID( C,R,L,V ) = AGRID( C,R,L,V )
     &                       + 2.0 * CGRID( C,R,K,S )
                     END DO
                  END DO
               END DO
            END DO
            
         END DO
         
         CALL SCIPROC ( CGRID, JDATE, JTIME, TSTEP, ASTEP )
        
c        write CGRID to CONC_CHK for final time step         
         CALL WR_CHK ( CGRID, JDATE, JTIME )

         DIVFAC = 0.5 / FLOAT( NREPS )

         DO V = 1, N_ASPCS
            S = AVG_CONC_MAP( V )
            L = 0
            DO K = ACONC_BLEV, ACONC_ELEV
               L = L + 1
               DO R = 1, MY_NROWS
                  DO C = 1, MY_NCOLS
                     AGRID( C,R,L,V ) = DIVFAC * ( AGRID( C,R,L,V )
     &                    +            CGRID( C,R,K,S ) )
                  END DO
               END DO
            END DO
         END DO

C     write conc fields

         CALL WR_CONC ( CGRID, JDATE, JTIME )

c$$$         CALL WR_ACONC ( AGRID, JDATE, JTIME, TSTEP( 1 ) )

         IF ( LIPR .OR. LIRR ) CALL PA_OUTPUT ( CGRID, JDATE, JTIME )

 201  CONTINUE

c---------------------------
c     Backward in Time Loop 
c---------------------------
      
      PRINT*,'--------------------------'
      PRINT*,'Backward loop starts here'
      PRINT*,'--------------------------'

      IF(LASTDAY)THEN
         LGRID = 0.000000

c        Initialize emission gradient variable to 0
         EMGRID = 0.000000
      
c        Initializes the adjoint variable at the receptor      
         CALL DEFINE_RECEPTOR( LGRID )

c        write LGRID to CONC_L5CHK for final time step  
         CALL WR_L5CHK ( LGRID, JDATE, JTIME )

         LASTDAY = .FALSE.

         print*,"complete the last day"

      ELSE

c        read the saved LGRID for initialization
         CALL RD_L5CHK ( LGRID, JDATE, JTIME )

c        read the saved EMGRID as starting point
         IF ( .NOT. READ3( CONC_L6CHK, ALLVAR3,
     &        000000, 000000, EMGRID ) ) THEN
            XMSG = 'Could not write EMGRID to '
     &           // CONC_L6CHK
            CALL M3EXIT( PNAME, JDATE, JTIME, XMSG, XSTAT1 )
         END IF         
         print*, "other days"
      END IF  !LASTDAY

c     read CGRID from CONC_CHK for final time step     
      CALL RD_CHK ( CGRID, JDATE, JTIME )

      RESDATE = JDATE
      RESTIME = JTIME
      
C     Main processing loop:

      DO 202 ISTEP = NSTEPS, 1, -1 ! output time step loop
C     Get synchronization and advection time steps, TSTEP(2), ASTEP(L) and NREPS
         SDATE = JDATE
         STIME = JTIME
         CALL NEXTIME(SDATE, STIME, -10000)
         
         CALL ADVSTEP ( SDATE, STIME, TSTEP, ASTEP, NREPS ) 
         
C     Reverse science process sequence:
         
         DO IREP = NREPS, 1, -1

            CALL NEXTIME ( RESDATE, RESTIME,-TSTEP( 2 ) )
            
c           read CGRID from CONC_CHK for dynamic time step                 
            CALL RD_CHK ( CGRID, RESDATE, RESTIME )

C           perform adjoint science process calculations on LGRID
            CALL SCIPROC_DADJ(LGRID, EMGRID, JDATE, JTIME, TSTEP, ASTEP)  

c           write LGRID to CONC_L5CHK for dynamic time step
            CALL WR_L5CHK ( LGRID, JDATE, JTIME )

         END DO
         
 202  CONTINUE

      IF ( .NOT. WRITE3( CONC_L6CHK, ALLVAR3,
     &     000000, 000000, EMGRID ) ) THEN
         XMSG = 'Could not write EMGRID to '
     &        // CONC_L6CHK
         CALL M3EXIT( PNAME, JDATE, JTIME, XMSG, XSTAT1 )
      END IF

      DEALLOCATE ( AGRID, STAT = ALLOCSTAT )                                            
      IF ( ALLOCSTAT .NE. 0 ) THEN
         XMSG = 'AGRID memory deallocation failed'
         CALL M3EXIT ( PNAME, JDATE, JTIME, XMSG, XSTAT1 )
      END IF

      DEALLOCATE ( EMGRID, STAT = ALLOCSTAT )
      IF ( ALLOCSTAT .NE. 0 ) THEN
         XMSG = 'EMGRID memory deallocation failed'
         CALL M3EXIT ( PNAME, JDATE, JTIME, XMSG, XSTAT1 )
      END IF

      DEALLOCATE ( ASTEP, STAT = ALLOCSTAT )
      IF ( ALLOCSTAT .NE. 0 ) THEN
         XMSG = 'ASTEP memory deallocation failed'
         CALL M3EXIT ( PNAME, JDATE, JTIME, XMSG, XSTAT1 )
      END IF
      
C     Clean up communications.
      
c$$$      CALL PAR_TERM ( PNAME, JDATE, JTIME, CLOCK )
      
      END SUBROUTINE
