   
C***********************************************************************
C   Portions of Models-3/CMAQ software were developed or based on      *
C   information from various groups: Federal Government employees,     *
C   contractors working on a United States Government contract, and    *
C   non-Federal sources (including research institutions).  These      *
C   research institutions have given the Government permission to      *
C   use, prepare derivative works, and distribute copies of their      *
C   work in Models-3/CMAQ to the public and to permit others to do     *
C   so.  EPA therefore grants similar permissions for use of the       *
C   Models-3/CMAQ software, but users are requested to provide copies  *
C   of derivative works to the Government without restrictions as to   *
C   use by others.  Users are responsible for acquiring their own      *
C   copies of commercial software associated with Models-3/CMAQ and    *
C   for complying with vendor requirements.  Software copyrights by    *
C   the MCNC Environmental Modeling Center are used with their         *
C   permissions subject to the above restrictions.                     *
C***********************************************************************

C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
      SUBROUTINE SCIPROC_DADJ ( LGRID, EMGRID, JDATE, JTIME, TSTEP, ASTEP )

C-----------------------------------------------------------------------
C Function:
C    ->Controls all of the physical and chemical adjoint processes for a grid
C    ->Operator splitting symmetric around chemistry
C    ->Designed for use with sensitivity analysis
 
C Preconditions:
C    Dates and times represented YYYYDDD:HHMMSS.
C    No "skipped" dates and times.  All boundary input variables (layered or
C    non-layered) have the same perimeter structure with a thickness of NTHIK
 
C Subroutines and functions called:
C    All physical and chemical subroutines, 
C    VDIFF_ADJ, XADV_DAD, YADV_DAD, ZADV_DAD, HDIFF_ADJ
C    ADJADV_ADJ, DECOUPLE, COUPLE
 
C Revision History:
C     Nov 08 T.Y. Gou & Sandu A.: built SCIPROC_DADJ subroutine to 
C           carry out science process adjoint calculations for sensitivity
C	    analysis
C-----------------------------------------------------------------------

      USE PCGRID_DEFN  ! inherits GRID_CONF and CGRID_SPCS
!     USE CGRID_SPCS   ! CGRID species number and offsets
      USE XFIRSTM

      IMPLICIT NONE   

C Include files:

!     INCLUDE SUBST_HGRD_ID     ! horizontal dimensioning parameters
!     INCLUDE SUBST_VGRD_ID     ! vertical dimensioning parameters
      INCLUDE SUBST_GC_SPC      ! gas chemistry species table
!     INCLUDE SUBST_AE_SPC      ! aerosol species table
!     INCLUDE SUBST_NR_SPC      ! non-reactive species table
!     INCLUDE SUBST_TR_SPC      ! tracer species table
      INCLUDE SUBST_IOPARMS     ! I/O parameters definitions
      INCLUDE SUBST_IODECL      ! I/O definitions and declarations
      INCLUDE SUBST_PACTL_ID    ! PA control parameters
      INCLUDE SUBST_FILES_ID    ! I/O definitions and declarations

C Arguments:
      REAL, POINTER :: EMGRID( :,:,:,: ) !Emission gradient variable
      REAL, POINTER :: LGRID( :,:,:,: )  !adjoint variable
      REAL, POINTER :: DBUFF( :,:,:,: )
      REAL, POINTER :: LGRID_B( :,:,: )  ! sensitivity to boundary conditions 

      INTEGER      JDATE        ! current model date, coded YYYYDDD
      INTEGER      JTIME        ! current model time, coded HHMMSS
      INTEGER      TSTEP( 2 )   ! time step vector (HHMMSS)
                                ! TSTEP(1) = local output step
                                ! TSTEP(2) = sciproc sync. step (chem)

!     INTEGER      ASTEP( NLAYS )  ! layer advection time step
      INTEGER      ASTEP( : )      ! layer advection time step

C Parameters:

C External Functions (not already declared by IODECL3.EXT):

      INTEGER, EXTERNAL :: SEC2TIME, TIME2SEC
      LOGICAL, EXTERNAL :: ENVYN

C Local Variables:

      CHARACTER( 16 ) :: PNAME = 'SCIPROC_DADJ'
      CHARACTER( 16 ) :: VNAME
      LOGICAL, SAVE :: FIRSTIME = .TRUE.
!      LOGICAL, SAVE :: XFIRST = .FALSE.

      CHARACTER( 120 ) :: XMSG = ' '
      CHARACTER( 36 ) :: NMSG = 'After NEXTIME: returned JDATE, JTIME'

      INTEGER, SAVE :: LOGDEV

      INTEGER      SDATE        ! current science process date, coded YYYYDDD
      INTEGER      STIME        ! current science process time, coded HHMMSS
 
      INTEGER     GXOFF, GYOFF  ! global origin offset from file
C     for INTERPX
      INTEGER, SAVE :: STRTCOLMC3, ENDCOLMC3, STRTROWMC3, ENDROWMC3
      
      INTEGER      ALLOCSTAT
      INTEGER      STATUS          ! ENV... status
      CHARACTER( 80 ) :: VARDESC   ! environment variable description
      CHARACTER( 80 ) :: MSG = ' '
      CHARACTER( 16 ) :: CTM_CKSUM = 'CTM_CKSUM'     ! env var for cksum on
!     LOGICAL, SAVE   :: CKSUM     ! flag for cksum on, default = [F]
      LOGICAL, SAVE   :: CKSUM     ! flag for cksum on, default = [T]

      INTERFACE
         SUBROUTINE CHEM_ADJ ( LGRID, JDATE, JTIME, TSTEP )
            IMPLICIT NONE
            REAL, POINTER             :: LGRID( :,:,:,: )
            INTEGER, INTENT( IN )     :: JDATE, JTIME
            INTEGER, INTENT( IN )     :: TSTEP( 2 )
         END SUBROUTINE CHEM_ADJ
         SUBROUTINE HDIFF_ADJ ( LGRID, JDATE, JTIME, TSTEP )
            IMPLICIT NONE
            REAL, POINTER             :: LGRID( :,:,:,: )
            INTEGER, INTENT( IN )     :: JDATE, JTIME
            INTEGER, INTENT( IN )     :: TSTEP( 2 )
         END SUBROUTINE HDIFF_ADJ
         SUBROUTINE ZADV_DAD ( LGRID, CGRID ,JDATE, JTIME, TSTEP )
            IMPLICIT NONE
            REAL, POINTER             :: LGRID( :,:,:,: )
            REAL, POINTER             :: CGRID( :,:,:,: )
            INTEGER, INTENT( IN )     :: JDATE, JTIME
            INTEGER, INTENT( IN )     :: TSTEP( 2 )
         END SUBROUTINE ZADV_DAD
         SUBROUTINE XADV_DAD ( LGRID, LGRID_B ,CHECK ,JDATE, JTIME, TSTEP, ASTEP )
            IMPLICIT NONE
            REAL, POINTER             :: LGRID( :,:,:,: )
            REAL, POINTER             :: CHECK( :,:,:,: )
            REAL, POINTER             :: LGRID_B(:,:,:)
            INTEGER, INTENT( IN )     :: JDATE, JTIME
            INTEGER, INTENT( IN )     :: TSTEP( 2 )
            INTEGER, INTENT( IN )     :: ASTEP( : )
         END SUBROUTINE XADV_DAD
         SUBROUTINE YADV_DAD ( LGRID,LGRID_B ,CHECK,JDATE, JTIME, TSTEP, ASTEP )
            IMPLICIT NONE
            REAL, POINTER             :: LGRID( :,:,:,: )
            REAL, POINTER             :: LGRID_B( :,:,: )
            REAL, POINTER             :: CHECK( :,:,:,: ) 
            INTEGER, INTENT( IN )     :: JDATE, JTIME
            INTEGER, INTENT( IN )     :: TSTEP( 2 )
            INTEGER, INTENT( IN )     :: ASTEP( : )
         END SUBROUTINE YADV_DAD
         SUBROUTINE ADJADV_ADJ (  CGRID,JDATE, JTIME, TSTEP )
            IMPLICIT NONE
            REAL, POINTER             :: CGRID( :,:,:,: )
            INTEGER, INTENT( IN )     :: JDATE, JTIME
            INTEGER, INTENT( IN )     :: TSTEP( 2 )
         END SUBROUTINE ADJADV_ADJ

         SUBROUTINE VDIFF_ADJ ( LGRID, EMGRID, JDATE, JTIME, TSTEP )
            IMPLICIT NONE
            REAL, POINTER             :: LGRID( :,:,:,: ), EMGRID( :,:,:,: )
            INTEGER, INTENT( IN )     :: JDATE, JTIME
            INTEGER, INTENT( IN )     :: TSTEP( 2 )
         END SUBROUTINE VDIFF_ADJ
         SUBROUTINE COUPLE ( CGRID, JDATE, JTIME, TSTEP )
            IMPLICIT NONE
            REAL, POINTER             :: CGRID( :,:,:,: )
            INTEGER, INTENT( IN )     :: JDATE, JTIME
            INTEGER, INTENT( IN )     :: TSTEP( 2 )
         END SUBROUTINE COUPLE
         SUBROUTINE DECOUPLE ( CGRID, JDATE, JTIME, TSTEP )
            IMPLICIT NONE
            REAL, POINTER             :: CGRID( :,:,:,: )
            INTEGER, INTENT( IN )     :: JDATE, JTIME
            INTEGER, INTENT( IN )     :: TSTEP( 2 )
         END SUBROUTINE DECOUPLE

      END INTERFACE
  
C-----------------------------------------------------------------------

      ALLOCATE ( DBUFF( MY_NCOLS,MY_NROWS,NLAYS,NSPCSD ), STAT = ALLOCSTAT )
      IF ( ALLOCSTAT .NE. 0 ) THEN
         XMSG = 'DBUFF memory allocation failed'
         CALL M3EXIT ( PNAME, JDATE, JTIME, XMSG, XSTAT1 )
      END IF

      ALLOCATE (LGRID_B(2*NCOLS+2*NROWS+4,NLAYS,NSPCSD), STAT = ALLOCSTAT)
      IF (ALLOCSTAT .NE. 0) THEN
         XMSG = 'LGRID_B memory allocation failed'
         CALL M3EXIT ( PNAME, JDATE, JTIME, XMSG, XSTAT1 )
      END IF
      
      SDATE = JDATE
      STIME = JTIME

      CALL NEXTIME ( JDATE, JTIME,-TSTEP( 2 ) )
      WRITE( LOGDEV,'(/ 5X, A, I8, I7.6)' ) NMSG, JDATE, JTIME

c$$$  CALL AERO ( CGRID, JDATE, JTIME, TSTEP )
         
      CALL CHEM_ADJ ( LGRID, JDATE, JTIME, TSTEP )
            
c$$$  CALL CLDPROC ( CGRID, JDATE, JTIME, TSTEP )
      

c$$$  CALL PING ( JDATE, JTIME, TSTEP )
  
      
C     Physical Processes for any Grid       
  
      CALL HDIFF_ADJ ( LGRID, JDATE, JTIME, TSTEP ) 



C ==============================================
C open CONC_L4CHK to read RHOJ for JDATE, JTIME

      IF (.NOT. READ3( CONC_L4CHK, ALLVAR3, ALLAYS3,
     &     JDATE, JTIME, DBUFF(:,:,:,:) ) ) THEN
         XMSG = 'Could not read RHOJ from ' // CONC_L4CHK
         CALL M3EXIT( PNAME, JDATE, JTIME, XMSG, XSTAT1 )
      END IF
      
!      LGRID(:,:,:,NSCPD) = DBUFF(:,:,:,NSCPD)
C ==============================================

      
      CALL DECOUPLE ( LGRID, JDATE, JTIME, TSTEP )
      IF ( CKSUM ) CALL CKSUMMER ( 'DECOUPLE', LGRID, JDATE, JTIME )
      
      IF ( N_GC_SPCD .GT. N_GC_SPC ) THEN
         CALL ADJADV_ADJ ( LGRID, JDATE, JTIME, TSTEP )
         
         IF ( CKSUM ) CALL CKSUMMER ( 'ADJADV', LGRID, JDATE, JTIME )
         IF ( LIPR ) CALL PA_UPDATE ( 'ADJC', LGRID, JDATE, JTIME, TSTEP )      
      END IF
    
      CALL ZADV_DAD  ( LGRID, DBUFF , JDATE, JTIME, TSTEP ) 
      
      print*,'---------------------------------'
      print*, 'XFIRST = ', XFIRST
      print*,'---------------------------------'
      
      IF ( XFIRST ) THEN
         XFIRST = .FALSE.
       
C ==============================================                                
C open CONC_XADVCHK to read conc for JDATE, JTIME                               

      IF ( .NOT. READ3( CONC_XADVCHK,ALLVAR3,ALLAYS3,
     &     JDATE, JTIME, DBUFF(:,:,:,:) ) ) THEN
         XMSG = 'Could not read CGRID to ' // CONC_XADVCHK
         CALL M3EXIT( PNAME, JDATE, JTIME, XMSG, XSTAT1 )
      END IF
      
C===============================================      
       
         CALL XADV_DAD ( LGRID, LGRID_B, DBUFF, JDATE, JTIME, TSTEP, ASTEP )

C ==============================================                                
C open CONC_YADVCHK to read conc for JDATE, JTIME                               
c$$$
      IF ( .NOT. READ3( CONC_YADVCHK,ALLVAR3,ALLAYS3,
     &        JDATE, JTIME, DBUFF(:,:,:,:) ) ) THEN
         XMSG = 'Could not read CGRID to ' // CONC_YADVCHK
         CALL M3EXIT( PNAME, JDATE, JTIME, XMSG, XSTAT1 )
      END IF
C===============================================           

      CALL YADV_DAD ( LGRID, LGRID_B ,DBUFF, JDATE, JTIME, TSTEP, ASTEP )
  
      ELSE
         XFIRST = .TRUE.
         
C==============================================                                
C open CONC_YADVCHK to read conc for JDATE, JTIME                               
c$$$
      IF ( .NOT. READ3( CONC_YADVCHK,ALLVAR3,ALLAYS3,
     &        JDATE, JTIME, DBUFF(:,:,:,:) ) ) THEN
         XMSG = 'Could not read CGRID to ' // CONC_YADVCHK
         CALL M3EXIT( PNAME, JDATE, JTIME, XMSG, XSTAT1 )
      END IF
c$$$C===============================================   
      CALL YADV_DAD ( LGRID, LGRID_B ,DBUFF , JDATE, JTIME, TSTEP, ASTEP )

C ==============================================                                
C open CONC_XADVCHK to read conc for JDATE, JTIME                               

      IF ( .NOT. READ3( CONC_XADVCHK,ALLVAR3,ALLAYS3,
     &     JDATE, JTIME, DBUFF(:,:,:,:) ) ) THEN
         XMSG = 'Could not read CGRID to ' // CONC_XADVCHK
         CALL M3EXIT( PNAME, JDATE, JTIME, XMSG, XSTAT1 )
      END IF
C=============================================== 
      
      CALL XADV_DAD ( LGRID, LGRID_B, DBUFF, JDATE, JTIME, TSTEP, ASTEP )
         
      END IF

     
      CALL COUPLE ( LGRID, JDATE, JTIME, TSTEP )                                                  
      IF ( CKSUM ) CALL CKSUMMER ( 'COUPLE', LGRID, JDATE, JTIME ) 
     
      CALL VDIFF_ADJ ( LGRID, EMGRID, JDATE, JTIME, TSTEP )
      
      DEALLOCATE ( DBUFF, STAT = ALLOCSTAT )
      IF ( ALLOCSTAT .NE. 0 ) THEN
         XMSG = 'DBUFF memory deallocation failed'
         CALL M3EXIT ( PNAME, JDATE, JTIME, XMSG, XSTAT1 )
      END IF

      DEALLOCATE(LGRID_B,STAT=ALLOCSTAT)
      IF (ALLOCSTAT .NE. 0) THEN
         XMSG = 'LGRID_B memory deallocation failed'
         CALL M3EXIT ( PNAME, JDATE, JTIME, XMSG, XSTAT1 )
      END IF


      RETURN
      END
